@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -903,17 +904,7 @@ bool Gcs_xcom_proxy_base::xcom_add_node(connection_descriptor &con,
 
 bool Gcs_xcom_proxy_base::test_xcom_tcp_connection(std::string &host,
                                                    xcom_port port) {
-  connection_descriptor *con = xcom_client_open_connection(host, port);
-
-  bool const could_connect_to_local_xcom = (con->fd != -1);
-  bool could_disconnect_from_local_xcom = false;
-  if (could_connect_to_local_xcom) {
-    could_disconnect_from_local_xcom = xcom_client_close_connection(con);
-  }
-
-  free(con);
-
-  return could_connect_to_local_xcom && could_disconnect_from_local_xcom;
+  return ::is_able_to_connect_to_node(host.c_str(), port);
 }
 
 bool Gcs_xcom_proxy_base::initialize_network_manager() {
