@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,6 +28,8 @@
 
 #include <assert.h>
 #include <stdlib.h>
+#include <algorithm>
+#include <iterator>
 
 #include "xcom/bitset.h"
 #include "xcom/node_list.h"
@@ -80,12 +83,22 @@ void init_site_vars() {
  the servers that it points to, since servers are shared by multiple
  site_defs, and will eventually be deallocated by garbage_collect_servers
 */
-void free_site_def(site_def *s) {
+
+void free_site_def_body(site_def *s) {
   if (s) {
     invalidate_detector_sites(s);
     xdr_free((xdrproc_t)xdr_node_list, (char *)(&s->nodes));
     free_node_set(&s->global_node_set);
     free_node_set(&s->local_node_set);
+    xdr_free((xdrproc_t)xdr_leader_array, (char *)(&s->leaders));
+    IFDBG(D_BUG, FN; STRLIT("free "); PTREXP(s); PTREXP(s->dispatch_table));
+    free(s->dispatch_table);
+  }
+}
+
+void free_site_def(site_def *s) {
+  if (s) {
+    free_site_def_body(s);
     free(s);
   }
 }
@@ -103,7 +116,7 @@ void free_site_defs() {
 /* Add a new site definition to the list */
 site_def *push_site_def(site_def *s) {
   uint32_t i;
-  set_site_def_ptr(&site_defs, 0, site_defs.count);
+  set_site_def_ptr(&site_defs, nullptr, site_defs.count);
   IFDBG(
       D_NONE, FN; NDBG(site_defs.count, u); PTREXP(s); if (s) {
         SYCEXP(s->start);
@@ -137,7 +150,7 @@ static inline site_def const *_get_site_def() {
   if (site_defs.count > 0)
     return site_defs.site_def_ptr_array_val[0];
   else
-    return 0;
+    return nullptr;
 }
 
 /* Return first site def */
@@ -145,7 +158,7 @@ site_def *get_site_def_rw() {
   if (site_defs.count > 0)
     return site_defs.site_def_ptr_array_val[0];
   else
-    return 0;
+    return nullptr;
 }
 
 /* purecov: begin deadcode */
@@ -157,7 +170,7 @@ static inline site_def const *_get_prev_site_def() {
   if (site_defs.count > 0)
     return site_defs.site_def_ptr_array_val[1];
   else
-    return 0;
+    return nullptr;
 }
 /* purecov: end */
 
@@ -180,7 +193,7 @@ static inline int match_def(site_def const *site, synode_no synode) {
 
 /* Return first site def which has start less than or equal to synode */
 site_def const *find_site_def(synode_no synode) {
-  site_def const *retval = 0;
+  site_def const *retval = nullptr;
   u_int i;
 
   for (i = 0; i < site_defs.count; i++)
@@ -195,7 +208,7 @@ site_def const *find_site_def(synode_no synode) {
 
 /* As find_site_def, but return pointer to non-const object */
 site_def *find_site_def_rw(synode_no synode) {
-  site_def *retval = 0;
+  site_def *retval = nullptr;
   u_int i;
 
   for (i = 0; i < site_defs.count; i++)
@@ -217,7 +230,7 @@ static inline int start_gt(site_def const *site, synode_no synode) {
 
 /* Retrieve the first site_def which has start greater than synode. */
 site_def const *find_next_site_def(synode_no synode) {
-  site_def const *retval = NULL;
+  site_def const *retval = nullptr;
   u_int i;
 
   for (i = site_defs.count; i > 0; i--)
@@ -225,7 +238,7 @@ site_def const *find_next_site_def(synode_no synode) {
       retval = site_defs.site_def_ptr_array_val[i - 1];
       break;
     }
-  assert(retval == NULL ||
+  assert(retval == nullptr ||
          retval->global_node_set.node_set_len == _get_maxnodes(retval));
   return retval;
 }
@@ -236,7 +249,7 @@ static inline int prev_def(site_def const *site, synode_no synode) {
 }
 
 site_def const *find_prev_site_def(synode_no synode) {
-  site_def const *retval = 0;
+  site_def const *retval = nullptr;
   u_int i;
 
   for (i = site_defs.count; i > 0; i--)
@@ -266,7 +279,7 @@ void garbage_collect_site_defs(synode_no x) {
     if (site) {
       IFDBG(D_NONE, SYCEXP(site->start); SYCEXP(site->boot_key););
       free_site_def(site);
-      site_defs.site_def_ptr_array_val[i] = 0;
+      site_defs.site_def_ptr_array_val[i] = nullptr;
     }
     site_defs.count--;
   }
@@ -281,11 +294,32 @@ char *dbg_site_def(site_def const *site) {
 
 /* Create a new empty site_def */
 site_def *new_site_def() {
-  site_def *retval = (site_def *)calloc((size_t)1, sizeof(site_def));
+  site_def *retval = (site_def *)xcom_calloc((size_t)1, sizeof(site_def));
   retval->nodeno = VOID_NODE_NO;
   return retval;
 }
 
+static void clone_leader(leader *l, leader const *x) {
+  l->address = strdup(x->address);
+}
+
+leader_array alloc_leader_array(u_int n) {
+  leader_array a{};
+  a.leader_array_val =
+      static_cast<leader *>(xcom_calloc((size_t)n, sizeof(leader)));
+  if (a.leader_array_val) a.leader_array_len = n;
+  return a;
+}
+
+leader_array clone_leader_array(leader_array const x) {
+  u_int i;
+  leader_array a = alloc_leader_array(x.leader_array_len);
+  for (i = 0; i < a.leader_array_len; i++) {
+    clone_leader(&a.leader_array_val[i], &x.leader_array_val[i]);
+  }
+  return a;
+}
+
 /* Clone a site definition */
 
 site_def *clone_site_def(site_def const *site) {
@@ -296,6 +330,9 @@ site_def *clone_site_def(site_def const *site) {
                  &retval->nodes);
   retval->global_node_set = clone_node_set(site->global_node_set);
   retval->local_node_set = clone_node_set(site->local_node_set);
+  retval->leaders = clone_leader_array(site->leaders);
+  retval->cached_leaders = false;    // Invalidate cached leaders
+  retval->dispatch_table = nullptr;  // Invalidate dispatch table
   assert(retval->global_node_set.node_set_len == _get_maxnodes(retval));
   IFDBG(D_NONE, FN; PTREXP(site); PTREXP(retval));
   return retval;
@@ -323,7 +360,7 @@ void init_site_def(u_int n, node_address *names, site_def *site) {
   site->x_proto = my_xcom_version;
   /* Inherit latest configuration's event horizon or fallback to default */
   latest_config = get_site_def();
-  if (latest_config != NULL) {
+  if (latest_config != nullptr) {
     site->event_horizon = latest_config->event_horizon;
   } else {
     site->event_horizon = EVENT_HORIZON_MIN;
@@ -344,10 +381,9 @@ void add_site_def(u_int n, node_address *names, site_def *site) {
 void remove_site_def(u_int n, node_address *names, site_def *site) {
   if (n > 0) {
     remove_node_list(n, names, &site->nodes);
+    realloc_node_set(&site->global_node_set, _get_maxnodes(site));
+    realloc_node_set(&site->local_node_set, _get_maxnodes(site));
   }
-  init_detector(site->detected); /* Zero all unused timestamps */
-  realloc_node_set(&site->global_node_set, _get_maxnodes(site));
-  realloc_node_set(&site->local_node_set, _get_maxnodes(site));
 }
 
 /* Return group id of site */
@@ -439,6 +475,8 @@ void import_config(gcs_snapshot *gcs_snap) {
         assert(cp->event_horizon);
         site->event_horizon = cp->event_horizon;
         copy_node_set(&cp->global_node_set, &site->global_node_set);
+        site->max_active_leaders = cp->max_active_leaders;
+        site->leaders = clone_leader_array(cp->leaders);
         site_install_action(site, app_type);
       }
     }
@@ -463,15 +501,15 @@ static synode_no get_conf_max() {
 gcs_snapshot *export_config() {
   u_int i;
   gcs_snapshot *gcs_snap =
-      (gcs_snapshot *)calloc((size_t)1, sizeof(gcs_snapshot));
+      (gcs_snapshot *)xcom_calloc((size_t)1, sizeof(gcs_snapshot));
   gcs_snap->cfg.configs_val =
-      (config_ptr *)calloc((size_t)site_defs.count, sizeof(config_ptr));
+      (config_ptr *)xcom_calloc((size_t)site_defs.count, sizeof(config_ptr));
   gcs_snap->cfg.configs_len = site_defs.count;
 
   for (i = 0; i < site_defs.count; i++) {
     site_def *site = site_defs.site_def_ptr_array_val[i];
     if (site) {
-      config_ptr cp = (config_ptr)calloc((size_t)1, sizeof(config));
+      config_ptr cp = (config_ptr)xcom_calloc((size_t)1, sizeof(config));
       init_node_list(site->nodes.node_list_len, site->nodes.node_list_val,
                      &cp->nodes);
       cp->start = site->start;
@@ -479,6 +517,8 @@ gcs_snapshot *export_config() {
       cp->event_horizon = site->event_horizon;
       assert(cp->event_horizon);
       cp->global_node_set = clone_node_set(site->global_node_set);
+      cp->max_active_leaders = site->max_active_leaders;
+      cp->leaders = clone_leader_array(site->leaders);
       IFDBG(D_BUG, FN; SYCEXP(cp->start); SYCEXP(cp->boot_key));
       gcs_snap->cfg.configs_val[i] = cp;
     }
@@ -544,21 +584,3 @@ synode_no get_highest_boot_key(gcs_snapshot *gcs_snap) {
   assert(!synode_eq(retval, null_synode));
   return retval;
 }
-
-/* Return boot_key of lowest numbered  config in snapshot */
-/* purecov: begin deadcode */
-synode_no get_lowest_boot_key(gcs_snapshot *gcs_snap) {
-  int i;
-  synode_no retval = null_synode;
-  IFDBG(D_NONE, FN; SYCEXP(gcs_snap->log_start); SYCEXP(gcs_snap->log_end));
-  for (i = (int)gcs_snap->cfg.configs_len - 1; i >= 0; i--) {
-    config_ptr cp = gcs_snap->cfg.configs_val[i];
-    if (cp) {
-      IFDBG(D_NONE, FN; SYCEXP(cp->start); SYCEXP(cp->boot_key));
-      retval = cp->boot_key;
-      break;
-    }
-  }
-  return retval;
-}
-/* purecov: end */
