@@ -37,6 +37,7 @@
 #include "mysql/harness/plugin_config.h"
 #include "mysql/harness/section_config_exposer.h"
 #include "mysql/harness/string_utils.h"
+#include "mysql/harness/supported_config_options.h"
 #include "mysql/harness/utility/string.h"  // join
 
 #ifdef _WIN32
@@ -183,7 +184,8 @@ class LoggingPluginConfig : public mysql_harness::BasePluginConfig {
     }
 
     if (sink_name == kFilelogPluginName) {
-      logging_folder = config.get_default("logging_folder");
+      logging_folder =
+          config.get_default(mysql_harness::loader::options::kLoggingFolder);
 
       if (logging_folder.empty()) {
         throw std::runtime_error(
@@ -214,13 +216,13 @@ class LoggingPluginConfig : public mysql_harness::BasePluginConfig {
   bool is_required(std::string_view /*option*/) const override { return false; }
 
   static constexpr const char *kLogLevel =
-      mysql_harness::logging::kConfigOptionLogLevel;
+      mysql_harness::logging::options::kLevel;
   static constexpr const char *kLogTimestampPrecision =
-      mysql_harness::logging::kConfigOptionLogTimestampPrecision;
+      mysql_harness::logging::options::kTimestampPrecision;
   static constexpr const char *kLogFilename =
-      mysql_harness::logging::kConfigOptionLogFilename;
+      mysql_harness::logging::options::kFilename;
   static constexpr const char *kDestination =
-      mysql_harness::logging::kConfigOptionLogDestination;
+      mysql_harness::logging::options::kDestination;
 };
 
 HandlerPtr create_logging_sink(const LoggingPluginConfig &config) {
