@@ -1,9 +1,8 @@
 var defaults = {
   version_comment: "community",
   account_user: "root",
-  metadata_schema_version: [2, 1, 0],
+  metadata_schema_version: [2, 2, 0],
   exec_time: 0.0,
-  group_replication_single_primary_mode: 1,
   // array-of-array
   // - server-uuid
   // - hostname
@@ -51,8 +50,18 @@ var defaults = {
         "role": "PRIMARY",
         "gr_uuid": "gr-id-1",
         "nodes": [
-          {"host": "127.0.0.1", "classic_port": 11010, "http_port": 11011},
-          {"host": "127.0.0.1", "classic_port": 11012, "http_port": 11013}
+          {
+            "host": "127.0.0.1",
+            "classic_port": 11010,
+            "http_port": 11011,
+            "attributes": "{}"
+          },
+          {
+            "host": "127.0.0.1",
+            "classic_port": 11012,
+            "http_port": 11013,
+            "attributes": "{}"
+          }
         ],
         "primary_node_id": 0
       },
@@ -63,8 +72,18 @@ var defaults = {
         "role": "SECONDARY",
         "gr_uuid": "gr-id-2",
         "nodes": [
-          {"host": "127.0.0.1", "classic_port": 11014, "http_port": 11015},
-          {"host": "127.0.0.1", "classic_port": 11016, "http_port": 11017}
+          {
+            "host": "127.0.0.1",
+            "classic_port": 11014,
+            "http_port": 11015,
+            "attributes": "{}"
+          },
+          {
+            "host": "127.0.0.1",
+            "classic_port": 11016,
+            "http_port": 11017,
+            "attributes": "{}"
+          }
         ],
         "primary_node_id": 0
       }
@@ -75,7 +94,6 @@ var defaults = {
   user_host_pattern: ".*",
   cluster_type: "gr",
   view_id: 1,
-  primary_port: 0,
   router_id: 1,
   // let the test that uses it set it explicitly, going with some default would
   // mean failures each time the version is bumped up (which we don't even
@@ -83,6 +101,7 @@ var defaults = {
   router_version: "",
   router_rw_classic_port: "",
   router_ro_classic_port: "",
+  router_rw_split_classic_port: "",
   router_rw_x_port: "",
   router_ro_x_port: "",
   router_metadata_user: "",
@@ -90,11 +109,12 @@ var defaults = {
   version: "8.0.24",  // SELECT @@version;
   router_expected_target_cluster: ".*",
   router_options: "",
-
   gr_member_state: "ONLINE",
   gr_members_all: 3,
   gr_members_recovering: 0,
   gr_members_online: 3,
+  current_instance_attributes: null,
+  config_defaults_stored_is_null: 0,
   last_insert_id: 1,
 };
 
@@ -171,7 +191,7 @@ function get_response(stmt_key, options) {
         stmt: "select @@port",
         result: {
           columns: [{name: "@@port", type: "LONG"}],
-          rows: [[options["port"]]]
+          rows: [[mysqld.session.port]]
         }
       };
     case "select_repeat_4097":
@@ -206,84 +226,65 @@ function get_response(stmt_key, options) {
           rows: [options["metadata_schema_version"]]
         }
       };
-    case "router_select_group_membership_with_primary_mode":
+    case "router_select_group_membership":
       return {
         stmt:
-            "SELECT member_id, member_host, member_port, member_state, @@group_replication_single_primary_mode FROM performance_schema.replication_group_members WHERE channel_name = 'group_replication_applier'",
+            "SELECT member_id, member_host, member_port, member_state, member_role, @@group_replication_single_primary_mode FROM performance_schema.replication_group_members WHERE channel_name = 'group_replication_applier'",
         exec_time: options["exec_time"],
         result: {
           columns: [
             {"name": "member_id", "type": "STRING"},
             {"name": "member_host", "type": "STRING"},
             {"name": "member_port", "type": "LONG"},
-            {"name": "member_state", "type": "STRING"}, {
+            {"name": "member_state", "type": "STRING"},
+            {"name": "member_role", "type": "STRING"},
+            {
               "name": "@@group_replication_single_primary_mode",
-              "type": "LONGLONG"
-            }
+              "type": "STRING"
+            },
           ],
           rows:
               options["group_replication_members"].map(function(currentValue) {
                 return [
-                  currentValue[0], currentValue[1], currentValue[2],
+                  currentValue[0],
+                  currentValue[1],
+                  currentValue[2],
                   currentValue[3],
-                  options["group_replication_single_primary_mode"]
+                  currentValue[4],
+                  "ON",
                 ];
               }),
         }
       };
-    case "router_select_group_replication_primary_member":
-      return {
-        "stmt": "show status like 'group_replication_primary_member'",
-        "result": {
-          "columns": [
-            {"name": "Variable_name", "type": "VAR_STRING"},
-            {"name": "Value", "type": "VAR_STRING"}
-          ],
-          "rows": [[
-            "group_replication_primary_member",
-            options["group_replication_primary_member"]
-          ]]
-        }
-      };
-    case "router_select_metadata":
+    case "router_select_group_membership_pre_8_0_2":
       return {
         stmt:
-            "SELECT F.cluster_id, F.cluster_name, R.replicaset_name, I.mysql_server_uuid, I.addresses->>'$.mysqlClassic', I.addresses->>'$.mysqlX' FROM mysql_innodb_cluster_metadata.clusters AS F JOIN mysql_innodb_cluster_metadata.replicasets AS R ON F.cluster_id = R.cluster_id JOIN mysql_innodb_cluster_metadata.instances AS I" +
-            " ON R.replicaset_id = I.replicaset_id" +
-            (options.gr_id === undefined || options.gr_id === "" ?
-                 " WHERE F.cluster_name = '" + options.innodb_cluster_name +
-                     "'" :
-                 " WHERE R.attributes->>'$.group_replication_group_name' = '" +
-                     options.gr_id + "'"),
+            "SELECT member_id, member_host, member_port, member_state, IF(g.primary_uuid = '' OR member_id = g.primary_uuid, 'PRIMARY', 'SECONDARY') as member_role, @@group_replication_single_primary_mode FROM (SELECT IFNULL(variable_value, '') AS primary_uuid FROM performance_schema.global_status WHERE variable_name = 'group_replication_primary_member') g, performance_schema.replication_group_members WHERE channel_name = 'group_replication_applier'",
         result: {
           columns: [
-            {"name": "F.cluster_id", "type": "VAR_STRING"},
-            {"name": "F.cluster_name", "type": "VAR_STRING"},
-            {"name": "R.replicaset_name", "type": "VAR_STRING"},
-            {"name": "I.mysql_server_uuid", "type": "VAR_STRING"},
-            {"name": "I.addresses->>'$.mysqlClassic'", "type": "LONGBLOB"},
-            {"name": "I.addresses->>'$.mysqlX'", "type": "LONGBLOB"}
+            {"name": "member_id", "type": "STRING"},
+            {"name": "member_host", "type": "STRING"},
+            {"name": "member_port", "type": "LONG"},
+            {"name": "member_state", "type": "STRING"},
+            {"name": "member_role", "type": "STRING"},
+            {
+              "name": "@@group_replication_single_primary_mode",
+              "type": "STRING"
+            },
           ],
-          rows: options["innodb_cluster_instances"].map(function(currentValue) {
-            var xport = currentValue[3] === undefined ? 0 : currentValue[3];
-            return [
-              options.cluster_id, options.innodb_cluster_name,
-              options.innodb_cluster_replicaset_name, currentValue[0],
-              currentValue[1] + ":" + currentValue[2],
-              currentValue[1] + ":" + xport
-            ]
-          }),
+          rows:
+              options["group_replication_members"].map(function(currentValue) {
+                return [
+                  currentValue[0],
+                  currentValue[1],
+                  currentValue[2],
+                  currentValue[3],
+                  currentValue[4],
+                  "ON",
+                ];
+              }),
         }
       };
-    case "router_select_metadata_account_verification":
-      return {
-        stmt:
-            "SELECT F.cluster_id, F.cluster_name, R.replicaset_name, I.mysql_server_uuid, I.addresses->>'$.mysqlClassic', I.addresses->>'$.mysqlX' FROM mysql_innodb_cluster_metadata.clusters AS F JOIN mysql_innodb_cluster_metadata.replicasets AS R ON F.cluster_id = R.cluster_id JOIN mysql_innodb_cluster_metadata.instances AS I" +
-            " ON R.replicaset_id = I.replicaset_id WHERE F.cluster_name = 'some_cluster_name'",
-        // The Router should ignore this result, it only checks if the user has
-        // rights to do it
-        result: {columns: [{"name": "1", "type": "LONG"}], rows: []}
-      };
     case "router_select_metadata_v2_gr":
       return {
         stmt:
@@ -339,17 +340,18 @@ function get_response(stmt_key, options) {
             {"name": "I.member_role", "type": "VAR_STRING"},
             {"name": "I.attributes", "type": "VAR_STRING"}
           ],
-          rows: options["innodb_cluster_instances"].map(function(currentValue) {
+          rows: options["innodb_cluster_instances"].map(function(
+              currentValue, index) {
             var xport = currentValue[3] === undefined ? 0 : currentValue[3];
             var attributes =
                 currentValue[4] === undefined ? "" : currentValue[4];
+            var default_role = index == 0 ? "PRIMARY" : "SECONDARY";
+            var role =
+                currentValue[5] === undefined ? default_role : currentValue[5];
             return [
               options.cluster_id, options.innodb_cluster_name, currentValue[0],
               currentValue[1] + ":" + currentValue[2],
-              currentValue[1] + ":" + xport,
-              currentValue[2] === options.primary_port ? "PRIMARY" :
-                                                         "SECONDARY",
-              attributes
+              currentValue[1] + ":" + xport, role, attributes
             ]
           }),
         }
@@ -425,7 +427,21 @@ function get_response(stmt_key, options) {
         },
       };
     case "router_start_transaction":
-      return {"stmt": "START TRANSACTION", "ok": {}};
+      return {
+        "stmt": "START TRANSACTION",
+        "ok": {
+          session_trackers: [
+            {
+              type: "trx_characteristics",
+              trx_stmt: "START TRANSACTION;",
+            },
+            {
+              type: "trx_state",
+              state: "________",
+            },
+          ]
+        }
+      };
     case "router_select_router_address":
       return {
         stmt_regex:
@@ -446,49 +462,6 @@ function get_response(stmt_key, options) {
           rows: [[options.router_id]]
         }
       };
-    case "router_select_hosts_v1":
-      return {
-        stmt_regex:
-            "^SELECT host_id, host_name, ip_address FROM mysql_innodb_cluster_metadata.hosts WHERE host_name = '" +
-            options.bootstrap_report_host_pattern + "' LIMIT 1",
-        result: {
-          columns: [
-            {"type": "STRING", "name": "host_id"},
-            {"type": "STRING", "name": "host_name"},
-            {"type": "STRING", "name": "ip_address"}
-          ],
-          rows: options["innodb_cluster_hosts"].map(function(currentValue) {
-            var xport = currentValue[2] === undefined ? 0 : currentValue[2];
-            return [currentValue[0], currentValue[1], currentValue[2]]
-          })
-        }
-      };
-    case "router_select_hosts_join_routers_v1":
-      return {
-        stmt_regex:
-            "SELECT h.host_id, h.host_name FROM mysql_innodb_cluster_metadata.routers r JOIN mysql_innodb_cluster_metadata.hosts h    ON r.host_id = h.host_id WHERE r.router_id = .*",
-        result: {
-          columns: [
-            {"type": "STRING", "name": "host_id"},
-            {"type": "STRING", "name": "host_name"}
-          ],
-          rows: options["innodb_cluster_hosts"].map(function(currentValue) {
-            return [currentValue[0], currentValue[1]]
-          })
-        }
-      };
-    case "router_insert_into_hosts_v1":
-      return {
-        "stmt_regex":
-            "^INSERT INTO mysql_innodb_cluster_metadata.hosts        \\(host_name, location, attributes\\) VALUES \\('" +
-            options.bootstrap_report_host_pattern + "',.*",
-        "ok": {"last_insert_id": 1}
-      };
-    case "router_insert_into_routers_v1":
-      return {
-        "stmt_regex": "^INSERT INTO mysql_innodb_cluster_metadata.routers.*",
-        "ok": {"last_insert_id": 1}
-      };
     case "router_insert_into_routers":
       return {
         "stmt_regex": "^INSERT INTO mysql_innodb_cluster_metadata.v2_routers.*",
@@ -507,7 +480,8 @@ function get_response(stmt_key, options) {
     case "router_create_user":
       return {
         "stmt_regex": "^CREATE USER 'mysql_router1_[0-9a-z]{12}'@" +
-            options.user_host_pattern + " IDENTIFIED BY '.*'",
+            options.user_host_pattern +
+            " IDENTIFIED WITH `caching_sha2_password` BY '.*'",
         "ok": {}
       };
     case "router_check_auth_plugin":
@@ -548,23 +522,13 @@ function get_response(stmt_key, options) {
             options.user_host_pattern,
         "ok": {}
       };
-    case "router_update_routers_in_metadata_v1":
-      return {
-        "stmt_regex":
-            "^UPDATE mysql_innodb_cluster_metadata\\.routers SET attributes =    " +
-            "JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(IF\\(attributes IS NULL, '\\{\\}', attributes\\),    " +
-            "'\\$\\.version', '.*'\\),    '\\$\\.RWEndpoint', '.*'\\),    '\\$\\.ROEndpoint', '.*'\\),    '\\$\\.RWXEndpoint', '.*'\\),    " +
-            "'\\$\\.ROXEndpoint', '.*'\\),    '\\$.MetadataUser', 'mysql_router.*'\\),    '\\$.bootstrapTargetType', '.*'\\) " +
-            "WHERE router_id = .*",
-        "ok": {}
-      };
     case "router_update_routers_in_metadata":
       return {
         "stmt_regex":
-            "^UPDATE mysql_innodb_cluster_metadata\\.v2_routers SET attributes =    " +
-            "JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(IF\\(attributes IS NULL, '\\{\\}', attributes\\),    " +
-            "'\\$\\.RWEndpoint', '.*'\\),    '\\$\\.ROEndpoint', '.*'\\),    '\\$\\.RWXEndpoint', '.*'\\),    " +
-            "'\\$\\.ROXEndpoint', '.*'\\),    '\\$\\.MetadataUser', '.*'\\),    '\\$\\.bootstrapTargetType', '.*'\\), " +
+            "^UPDATE mysql_innodb_cluster_metadata\\.v2_routers SET attributes = " +
+            "JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(IF\\(attributes IS NULL, '\\{\\}', attributes\\),    " +
+            "'\\$\\.RWEndpoint', '.*'\\),    '\\$\\.ROEndpoint', '.*'\\),    '\\$\\.RWSplitEndpoint', '.*'\\),    '\\$\\.RWXEndpoint', '.*'\\),    " +
+            "'\\$\\.ROXEndpoint', '.*'\\),    '\\$\\.MetadataUser', '.*'\\),    '\\$\\.bootstrapTargetType', '.*'\\),    '\\$\\.Configuration', CAST\\('(.*)' as JSON\\)\\), " +
             "version = '.*', cluster_id = '.*' " +
             "WHERE router_id = .*",
         "ok": {}
@@ -572,10 +536,10 @@ function get_response(stmt_key, options) {
     case "router_clusterset_update_routers_in_metadata":
       return {
         "stmt_regex":
-            "^UPDATE mysql_innodb_cluster_metadata\\.v2_routers SET attributes =    " +
-            "JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(IF\\(attributes IS NULL, '\\{\\}', attributes\\),    " +
-            "'\\$\\.RWEndpoint', '.*'\\),    '\\$\\.ROEndpoint', '.*'\\),    '\\$\\.RWXEndpoint', '.*'\\),    " +
-            "'\\$\\.ROXEndpoint', '.*'\\),    '\\$\\.MetadataUser', '.*'\\),    '\\$\\.bootstrapTargetType', '.*'\\), " +
+            "^UPDATE mysql_innodb_cluster_metadata\\.v2_routers SET attributes = " +
+            "JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(IF\\(attributes IS NULL, '\\{\\}', attributes\\),    " +
+            "'\\$\\.RWEndpoint', '.*'\\),    '\\$\\.ROEndpoint', '.*'\\),    '\\$\\.RWSplitEndpoint', '.*'\\),    '\\$\\.RWXEndpoint', '.*'\\),    " +
+            "'\\$\\.ROXEndpoint', '.*'\\),    '\\$\\.MetadataUser', '.*'\\),    '\\$\\.bootstrapTargetType', '.*'\\),    '\\$\\.Configuration', CAST\\('(.*)' as JSON\\)\\), " +
             "version = '.*', clusterset_id = '.*' " +
             "WHERE router_id = .*",
         "ok": {}
@@ -590,9 +554,41 @@ function get_response(stmt_key, options) {
         "ok": {}
       };
     case "router_commit":
-      return {stmt: "COMMIT", ok: {}};
+      return {
+        stmt: "COMMIT",
+        ok: {
+          session_trackers: [
+            {
+              type: "trx_characteristics",
+              value: "",
+            },
+            {
+              type: "trx_state",
+              state: "________",
+            },
+            {
+              type: "gtid",
+              gtid: "3E11FA47-71CA-11E1-9E33-C80AA9429562:23",
+            }
+          ]
+        }
+      };
     case "router_rollback":
-      return {stmt: "ROLLBACK", ok: {}};
+      return {
+        stmt: "ROLLBACK",
+        ok: {
+          session_trackers: [
+            {
+              type: "trx_characteristics",
+              value: "",
+            },
+            {
+              type: "trx_state",
+              state: "________",
+            },
+          ]
+        }
+      };
     case "router_replication_group_members":
       return {
         stmt:
@@ -613,45 +609,6 @@ function get_response(stmt_key, options) {
       };
     case "router_drop_users":
       return {stmt_regex: "^DROP USER IF EXISTS 'mysql_router.*", ok: {}};
-    case "router_select_metadata_v2":
-      return {
-        stmt:
-            "select 'default' as replicaset_name, I.mysql_server_uuid, I.endpoint, I.xendpoint, I.attributes from " +
-            "mysql_innodb_cluster_metadata.v2_instances I join " +
-            "mysql_innodb_cluster_metadata.v2_gr_clusters C on I.cluster_id = " +
-            "C.cluster_id where C.cluster_name =  " +
-            options.innodb_cluster_name + "'" +
-            (options.gr_id === undefined || options.gr_id === "" ?
-                 "" :
-                 (" AND R.attributes->>'$.group_replication_group_name' = '" +
-                  options.gr_id + "'")),
-        result: {
-          columns: [
-            {"name": "replicaset_name", "type": "VAR_STRING"},
-            {"name": "mysql_server_uuid", "type": "VAR_STRING"},
-            {"name": "I.addresses->>'$.mysqlClassic'", "type": "LONGBLOB"},
-            {"name": "I.addresses->>'$.mysqlX'", "type": "LONGBLOB"},
-            {"name": "I.attributes", "type": "VAR_STRING"}
-          ],
-          rows: options["innodb_cluster_instances"].map(function(currentValue) {
-            var xport = currentValue[3] === undefined ? 0 : currentValue[3];
-            var attributes =
-                currentValue[4] === undefined ? "" : currentValue[4];
-            return [
-              options.innodb_cluster_replicaset_name, currentValue[0],
-              currentValue[1] + ":" + currentValue[2],
-              currentValue[1] + ":" + xport, attributes
-            ]
-          }),
-        }
-      };
-    case "router_count_clusters_v1":
-      return {
-        stmt: "select count(*) from " +
-            "mysql_innodb_cluster_metadata.clusters",
-        result:
-            {columns: [{"type": "LONGLONG", "name": "count(*)"}], rows: [[1]]}
-      };
     case "router_count_clusters_v2":
       return {
         stmt: "select count(*) from " +
@@ -688,43 +645,10 @@ function get_response(stmt_key, options) {
           "rows": [["Mysqlx_ssl_cipher", mysqld.session.mysqlx_ssl_cipher]]
         }
       };
-    case "router_select_cluster_instances_v1":
-      return {
-        "stmt":
-            "SELECT F.cluster_id, R.attributes->>'$.group_replication_group_name' as uuid, F.cluster_name, " +
-            "JSON_UNQUOTE(JSON_EXTRACT(I.addresses, '$.mysqlClassic')) " +
-            "FROM " +
-            "mysql_innodb_cluster_metadata.clusters AS F, " +
-            "mysql_innodb_cluster_metadata.instances AS I, " +
-            "mysql_innodb_cluster_metadata.replicasets AS R " +
-            "WHERE R.replicaset_id = (SELECT replicaset_id FROM mysql_innodb_cluster_metadata.instances " +
-            "WHERE CAST(mysql_server_uuid AS char ascii) = CAST(@@server_uuid AS char ascii)) AND I.replicaset_id = R.replicaset_id " +
-            "AND R.cluster_id = F.cluster_id",
-        result: {
-          columns: [
-            {"type": "STRING", "name": "cluster_id"},
-            {"type": "STRING", "name": "uuid"},
-            {"type": "STRING", "name": "cluster_name"},
-            {
-              "type": "STRING",
-              "name":
-                  "JSON_UNQUOTE(JSON_EXTRACT(I.addresses, '$.mysqlClassic'))"
-            },
-          ],
-          rows: options["innodb_cluster_instances"].map(function(currentValue) {
-            return [
-              options.cluster_id,
-              options.gr_id,
-              options.innodb_cluster_name,
-              currentValue[1] + ":" + currentValue[2],
-            ]
-          })
-        }
-      };
     case "router_select_cluster_instances_v2_ar":
       return {
         "stmt":
-            "select c.cluster_id, c.cluster_id as uuid, c.cluster_name, i.address from " +
+            "select c.cluster_id, c.cluster_id as uuid, c.cluster_name, i.address, i.attributes from " +
             "mysql_innodb_cluster_metadata.v2_instances i join " +
             "mysql_innodb_cluster_metadata.v2_clusters c on c.cluster_id = " +
             "i.cluster_id",
@@ -734,13 +658,17 @@ function get_response(stmt_key, options) {
             {"type": "STRING", "name": "uuid"},
             {"type": "STRING", "name": "cluster_name"},
             {"type": "STRING", "name": "i.address"},
+            {"type": "STRING", "name": "i.attributes"}
           ],
           rows: options["innodb_cluster_instances"].map(function(currentValue) {
+            var attributes =
+                currentValue[4] === undefined ? "" : currentValue[4];
             return [
               options.cluster_id,
               options.cluster_id,
               options.innodb_cluster_name,
               currentValue[1] + ":" + currentValue[2],
+              attributes,
             ]
           })
         }
@@ -748,7 +676,7 @@ function get_response(stmt_key, options) {
     case "router_select_cluster_instances_v2_gr":
       return {
         "stmt":
-            "select c.cluster_id, c.group_name as uuid, c.cluster_name, i.address from " +
+            "select c.cluster_id, c.group_name as uuid, c.cluster_name, i.address, i.attributes from " +
             "mysql_innodb_cluster_metadata.v2_instances i join " +
             "mysql_innodb_cluster_metadata.v2_gr_clusters c on c.cluster_id = " +
             "i.cluster_id",
@@ -758,13 +686,17 @@ function get_response(stmt_key, options) {
             {"type": "STRING", "name": "uuid"},
             {"type": "STRING", "name": "cluster_name"},
             {"type": "STRING", "name": "i.address"},
+            {"type": "STRING", "name": "i.attributes"},
           ],
           rows: options["innodb_cluster_instances"].map(function(currentValue) {
+            var attributes =
+                currentValue[4] === undefined ? "" : currentValue[4];
             return [
               options.cluster_id,
               options.gr_id,
               options.innodb_cluster_name,
               currentValue[1] + ":" + currentValue[2],
+              attributes,
             ]
           })
         }
@@ -799,7 +731,8 @@ function get_response(stmt_key, options) {
       return {
         "stmt_regex": "^CREATE USER IF NOT EXISTS '" +
             options.account_user_pattern + "'@'" +
-            options.account_host_pattern + "' IDENTIFIED BY '" +
+            options.account_host_pattern +
+            "' IDENTIFIED WITH `caching_sha2_password` BY '" +
             options.account_pass_pattern + "'",
         "ok": {warning_count: options.create_user_warning_count}
       };
@@ -808,7 +741,8 @@ function get_response(stmt_key, options) {
         // CREATE USER (without IF NOT EXISTS) is triggered by
         // --account-create always
         "stmt_regex": "^CREATE USER '" + options.account_user_pattern + "'@'" +
-            options.account_host_pattern + "' IDENTIFIED BY '" +
+            options.account_host_pattern +
+            "' IDENTIFIED WITH `caching_sha2_password` BY '" +
             options.account_pass_pattern + "'",
         "ok": {}
       };
@@ -821,31 +755,6 @@ function get_response(stmt_key, options) {
           "rows": [[options.cluster_id]]
         }
       };
-    case "router_update_attributes_v1":
-      return {
-        "stmt_regex": "UPDATE mysql_innodb_cluster_metadata\\.routers" +
-            " SET attributes = JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(" +
-            " IF\\(attributes IS NULL, '\\{\\}', attributes\\), '\\$\\.version', '.*'\\)," +
-            " '\\$\\.RWEndpoint', '.*'\\), '\\$\\.ROEndpoint', '.*'\\), '\\$\\.RWXEndpoint', '.*'\\)," +
-            " '\\$\\.ROXEndpoint', '.*'\\), '\\$\\.MetadataUser', '.*'\\) WHERE router_id = " +
-            options.router_id,
-        "ok": {}
-      };
-    case "router_update_attributes_strict_v1":
-      // the exact match, not regex
-      return {
-        "stmt": "UPDATE mysql_innodb_cluster_metadata.routers" +
-            " SET attributes = JSON_SET(JSON_SET(JSON_SET(JSON_SET(JSON_SET(JSON_SET(" +
-            " IF(attributes IS NULL, '{}', attributes)," +
-            " '$.version', '" + options.router_version +
-            "'), '$.RWEndpoint', '" + options.router_rw_classic_port +
-            "'), '$.ROEndpoint', '" + options.router_ro_classic_port +
-            "'), '$.RWXEndpoint', '" + options.router_rw_x_port +
-            "'), '$.ROXEndpoint', '" + options.router_ro_x_port +
-            "'), '$.MetadataUser', '" + options.router_metadata_user +
-            "') WHERE router_id = " + options.router_id,
-        "ok": {}
-      };
     case "router_create_user_show_warnings":
       // this query will only be issued if CREATE USER [IF NOT EXISTS]
       // returned warning_count > 0
@@ -869,25 +778,10 @@ function get_response(stmt_key, options) {
     case "router_update_attributes_v2":
       return {
         "stmt_regex": "UPDATE mysql_innodb_cluster_metadata\\.v2_routers" +
-            " SET version = .*, last_check_in = NOW\\(\\), attributes = JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(" +
+            " SET version = '(.*)', last_check_in = NOW\\(\\), attributes = JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(JSON_SET\\(" +
             " IF\\(attributes IS NULL, '\\{\\}', attributes\\)," +
-            " '\\$\\.RWEndpoint', '.*'\\), '\\$\\.ROEndpoint', '.*'\\), '\\$\\.RWXEndpoint', '.*'\\)," +
-            " '\\$\\.ROXEndpoint', '.*'\\), '\\$\\.MetadataUser', '.*'\\) WHERE router_id = .*",
-        "ok": {}
-      };
-    case "router_update_attributes_strict_v2":
-      return {
-        "stmt": "UPDATE mysql_innodb_cluster_metadata.v2_routers" +
-            " SET version = '" + options.router_version +
-            "', last_check_in = NOW()" +
-            ", attributes = JSON_SET(JSON_SET(JSON_SET(JSON_SET(JSON_SET(" +
-            " IF(attributes IS NULL, '{}', attributes)," +
-            " '$.RWEndpoint', '" + options.router_rw_classic_port +
-            "'), '$.ROEndpoint', '" + options.router_ro_classic_port +
-            "'), '$.RWXEndpoint', '" + options.router_rw_x_port +
-            "'), '$.ROXEndpoint', '" + options.router_ro_x_port +
-            "'), '$.MetadataUser', '" + options.router_metadata_user +
-            "') WHERE router_id = " + options.router_id,
+            " '\\$\\.RWEndpoint', '(.*)'\\), '\\$\\.ROEndpoint', '(.*)'\\), '\\$\\.RWSplitEndpoint', '(.*)'\\), '\\$\\.RWXEndpoint', '(.*)'\\)," +
+            " '\\$\\.ROXEndpoint', '(.*)'\\), '\\$\\.MetadataUser', '(.*)'\\), '\\$.Configuration', CAST\\('(.*)' as JSON\\)\\) WHERE router_id = (.*)",
         "ok": {}
       };
     case "router_update_last_check_in_v2":
@@ -1064,7 +958,7 @@ function get_response(stmt_key, options) {
       };
     case "router_clusterset_all_nodes":
       return {
-        stmt: "SELECT i.address, csm.member_role " +
+        stmt: "SELECT i.address, i.attributes, csm.member_role " +
             "FROM mysql_innodb_cluster_metadata.v2_instances i " +
             "LEFT JOIN mysql_innodb_cluster_metadata.v2_cs_members csm " +
             "ON i.cluster_id = csm.cluster_id " +
@@ -1082,6 +976,7 @@ function get_response(stmt_key, options) {
         result: {
           columns: [
             {"type": "STRING", "name": "i.address"},
+            {"type": "STRING", "name": "i.attributes"},
             {"type": "STRING", "name": "csm.member_role"},
           ],
 
@@ -1097,7 +992,8 @@ function get_response(stmt_key, options) {
                         [])
                     .map(function(node) {
                       return [
-                        node.host + ":" + node.classic_port, node.cluster_role
+                        node.host + ":" + node.classic_port, node.attributes,
+                        node.cluster_role
                       ]
                     })
         }
@@ -1145,9 +1041,8 @@ function get_response(stmt_key, options) {
                         node.uuid, node.host + ":" + node.classic_port,
                         node.host + ":" +
                             (node.x_port === undefined ? 0 : node.x_port),
-                        "",  // is this ok ?
-                        node.cluster_uuid, node.cluster_name, node.cluster_role,
-                        node.cluster_invalid,
+                        node.attributes, node.cluster_uuid, node.cluster_name,
+                        node.cluster_role, node.cluster_invalid,
                         options.clusterset_data.clusterset_name
                       ];
                     })
@@ -1223,10 +1118,10 @@ function get_response(stmt_key, options) {
           rows: [[options.bootstrap_target_type]]
         }
       };
-    case "router_router_options":
+    case "router_router_select_cs_options":
       return {
         stmt:
-            "SELECT router_options FROM mysql_innodb_cluster_metadata.v2_cs_router_options where router_id = " +
+            "SELECT router_options FROM mysql_innodb_cluster_metadata.v2_cs_router_options WHERE router_id = " +
             options.router_id,
         result: {
           columns: [{"name": "router_options", "type": "VAR_STRING"}],
@@ -1306,58 +1201,119 @@ function get_response(stmt_key, options) {
           rows: []
         }
       };
-    case "router_clusterset_select_gr_primary_member":
-      return {
-        stmt: "show status like 'group_replication_primary_member'",
-        result: {
-          columns: [
-            {"name": "Variable_name", "type": "VAR_STRING"},
-            {"name": "Value", "type": "VAR_STRING"}
-          ],
-          rows:
-              [options.clusterset_data
-                       .clusters[options.clusterset_data.this_cluster_id]
-                       .nodes[options.clusterset_data
-                                  .clusters[options.clusterset_data
-                                                .this_cluster_id]
-                                  .primary_node_id] ?
-                   [
-                     "group_replication_primary_member",
-                     options.clusterset_data
-                         .clusters[options.clusterset_data.this_cluster_id]
-                         .nodes[options.clusterset_data
-                                    .clusters[options.clusterset_data
-                                                  .this_cluster_id]
-                                    .primary_node_id]
-                         .uuid
-                   ] :
-                   []]
-        }
-      };
     case "router_clusterset_select_gr_members_status":
       return {
         stmt:
-            "SELECT member_id, member_host, member_port, member_state, @@group_replication_single_primary_mode FROM performance_schema.replication_group_members WHERE channel_name = 'group_replication_applier'",
-            result: {
-              columns:
-                  [
-                    {"name": "member_id", "type": "STRING"},
-                    {"name": "member_host", "type": "STRING"},
-                    {"name": "member_port", "type": "LONG"},
-                    {"name": "member_state", "type": "STRING"}, {
-                      "name": "@@group_replication_single_primary_mode",
-                      "type": "LONGLONG"
-                    }
-                  ],
-              rows: options.clusterset_data
-                      .clusters[options.clusterset_data.this_cluster_id]
-                      .nodes.map(function(node) {
-                        return [
-                          node.uuid, node.host, node.classic_port, "ONLINE", 1
-                        ];
-                      }),
-            }
-      }
+            "SELECT member_id, member_host, member_port, member_state, member_role, @@group_replication_single_primary_mode FROM performance_schema.replication_group_members WHERE channel_name = 'group_replication_applier'",
+        result: {
+          columns: [
+            {"name": "member_id", "type": "STRING"},
+            {"name": "member_host", "type": "STRING"},
+            {"name": "member_port", "type": "LONG"},
+            {"name": "member_state", "type": "STRING"},
+            {"name": "member_role", "type": "STRING"},
+            {
+              "name": "@@group_replication_single_primary_mode",
+              "type": "STRING"
+            },
+          ],
+          rows: options.clusterset_data
+                    .clusters[options.clusterset_data.this_cluster_id]
+                    .gr_nodes.map(function(node) {
+                      return [
+                        node.uuid, "127.0.0.1", node.classic_port, node.status,
+                        node.role, "ON"
+                      ];
+                    }),
+        }
+      };
+    case "router_select_router_options_view":
+      return {
+        stmt:
+            "SELECT router_options FROM mysql_innodb_cluster_metadata.v2_router_options WHERE router_id = " +
+            options.router_id,
+        result: {
+          columns: [{"type": "STRING", "name": "router_options"}],
+          rows: [[options.router_options]]
+        }
+      };
+    case "router_select_current_instance_attributes":
+      return {
+        stmt:
+            "select i.attributes from mysql_innodb_cluster_metadata.v2_this_instance " +
+            "ti left join mysql_innodb_cluster_metadata.v2_instances i " +
+            "on ti.instance_id = i.instance_id",
+        result: {
+          columns: [{"type": "STRING", "name": "attributes"}],
+          rows: [[options.current_instance_attributes]]
+        }
+      };
+    case "router_select_config_defaults_stored_gr_cluster":
+      return {
+        stmt_regex:
+            "select JSON_EXTRACT\\(router_options, '\\$\\.Configuration.\"" +
+            options["router_version"] +
+            "\"'\\) IS NULL from mysql_innodb_cluster_metadata.v2_gr_clusters where cluster_id = '.*'",
+        result: {
+          columns: [{
+            "type": "LONGLONG",
+            "name": "JSON_EXTRACT(router_options, '$.Configuration\"') IS NULL"
+          }],
+          rows: [[options.config_defaults_stored_is_null]]
+        }
+      };
+    case "router_select_config_defaults_stored_ar_cluster":
+      return {
+        stmt_regex:
+            "select JSON_EXTRACT\\(router_options, '\\$\\.Configuration.\"" +
+            options["router_version"] +
+            "\"'\\) IS NULL from mysql_innodb_cluster_metadata.v2_ar_clusters where cluster_id = '.*'",
+        result: {
+          columns: [{
+            "type": "LONGLONG",
+            "name": "JSON_EXTRACT(router_options, '$.Configuration') IS NULL"
+          }],
+          rows: [[options.config_defaults_stored_is_null]]
+        }
+      };
+    case "router_select_config_defaults_stored_clusterset":
+      return {
+        stmt_regex:
+            "select JSON_EXTRACT\\(router_options, '\\$\\.Configuration.\"" +
+            options["router_version"] +
+            "\"'\\) IS NULL from mysql_innodb_cluster_metadata.v2_cs_clustersets where clusterset_id = '.*'",
+        result: {
+          columns: [{
+            "type": "LONGLONG",
+            "name": "JSON_EXTRACT(router_options, '$.Configuration') IS NULL"
+          }],
+          rows: [[options.config_defaults_stored_is_null]]
+        }
+      };
+    case "router_store_config_defaults_gr_cluster":
+      return {
+        stmt_regex:
+            "UPDATE mysql_innodb_cluster_metadata\\.v2_gr_clusters SET router_options = " +
+            "JSON_MERGE_PATCH\\(IF\\(router_options IS NULL, '\\{\\}', router_options\\), " +
+            "CAST\\('(.*)' as JSON\\)\\) WHERE cluster_id = '.*'",
+        ok: {}
+      };
+    case "router_store_config_defaults_ar_cluster":
+      return {
+        stmt_regex:
+            "UPDATE mysql_innodb_cluster_metadata\\.v2_ar_clusters SET router_options = " +
+            "JSON_MERGE_PATCH\\(IF\\(router_options IS NULL, '\\{\\}', router_options\\), " +
+            "CAST\\('(.*)' as JSON\\)\\) WHERE cluster_id = '.*'",
+        ok: {}
+      };
+    case "router_store_config_defaults_clusterset":
+      return {
+        stmt_regex:
+            "UPDATE mysql_innodb_cluster_metadata\\.v2_cs_clustersets SET router_options = " +
+            "JSON_MERGE_PATCH\\(IF\\(router_options IS NULL, '\\{\\}', router_options\\), " +
+            "CAST\\('(.*)' as JSON\\)\\) WHERE clusterset_id = '.*'",
+        ok: {}
+      };
   };
 };
 
@@ -1438,7 +1394,7 @@ exports.unknown_statement_response = function(stmt) {
  * @param {string} regex_stmt regex statement text
  * @param {object} common_responses_regex object containing
  *     common responses
- * @returns  response if any matches the statmenet or undefined
+ * @returns  response if any matches the statement or undefined
  */
 exports.handle_regex_stmt = function(regex_stmt, common_responses_regex) {
   for (var stmt in common_responses_regex) {
