@@ -1,16 +1,17 @@
 /*
- Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+ Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is also distributed with certain software (including
+ This program is designed to work with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have included with MySQL.
+ separately licensed software that they have either included with
+ the program or referenced in the documentation.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -89,7 +90,6 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
         "--bootstrap=" + gr_member_ip + ":" + std::to_string(cluster_node_port),
         "-d",
         temp_test_dir.name(),
-        "--conf-set-option=DEFAULT.logging_folder=" + get_logging_dir().str(),
         "--conf-set-option=logger.level=DEBUG",
     };
 
@@ -105,6 +105,27 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
                           std::to_string(router_port_x_rw),
                       "--conf-set-option=routing:bootstrap_x_ro.bind_port=" +
                           std::to_string(router_port_x_ro)});
+
+      // Let's overwrite the default bind_address to prevent the MacOS firewall
+      // from complaining
+      cmdline.insert(
+          cmdline.end(),
+          {"--conf-set-option=DEFAULT.bind_address=127.0.0.1",
+           "--conf-set-option=routing:bootstrap_rw.bind_address=127.0.0.1",
+           "--conf-set-option=routing:bootstrap_ro.bind_address=127.0.0.1",
+           "--conf-set-option=routing:bootstrap_x_rw.bind_address=127.0.0.1",
+           "--conf-set-option=routing:bootstrap_x_ro.bind_address=127.0.0.1"});
+
+      if (std::find(additional_config.begin(), additional_config.end(),
+                    "--disable-rw-split") == additional_config.end()) {
+        // if --disable-rw-split isn't set, set the bind-port and bind-address
+        cmdline.insert(
+            cmdline.end(),
+            {"--conf-set-option=routing:bootstrap_rw_split.bind_port=" +
+                 std::to_string(router_port_rw_split),
+             "--conf-set-option=routing:bootstrap_rw_split.bind_address="
+             "127.0.0.1"});
+      }
     }
 
     std::move(std::begin(additional_config), std::end(additional_config),
@@ -117,10 +138,6 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
     EXPECT_TRUE(router_bootstrap.expect_output(
         "MySQL Router configured for the InnoDB Cluster 'mycluster'"));
 
-    auto plugin_dir = mysql_harness::get_plugin_dir(get_origin().str());
-    EXPECT_TRUE(add_line_to_config_file(config_path.str(), "DEFAULT",
-                                        "plugin_folder", plugin_dir));
-
     return router_bootstrap;
   }
 
@@ -258,7 +275,8 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
   }
 
   void assert_rest_works(const uint16_t port) {
-    const auto uri = std::string(rest_api_basepath) + "/router/status";
+    const auto uri = "https://" + gr_member_ip + ":" + std::to_string(port) +
+                     rest_api_basepath + "/router/status";
 
     const auto ca_file =
         datadir_path.join(cert_filenames.at(CertFile::k_ca_cert));
@@ -278,9 +296,7 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
     }
 
     IOContext io_ctx;
-    auto https_client = std::make_unique<HttpsClient>(
-        io_ctx, std::move(tls_ctx), gr_member_ip, port);
-    RestClient rest_client(std::move(https_client));
+    RestClient rest_client(io_ctx, std::move(tls_ctx));
 
     JsonDocument json_doc;
     // We do not care to authenticate, just check if we got a response
@@ -363,6 +379,7 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
   uint16_t custom_port;
   uint16_t router_port_rw;
   uint16_t router_port_ro;
+  uint16_t router_port_rw_split;
   uint16_t router_port_x_rw;
   uint16_t router_port_x_ro;
   ProcessWrapper *cluster_node;
@@ -390,8 +407,9 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
 
  protected:
   void set_globals(std::string cluster_id = "") {
-    set_mock_metadata(cluster_http_port, cluster_id, {cluster_node_port}, 0,
-                      {cluster_node_port}, 0 /*primary_id*/, 0 /*view_id*/,
+    set_mock_metadata(cluster_http_port, cluster_id,
+                      classic_ports_to_gr_nodes({cluster_node_port}), 0,
+                      {cluster_node_port}, 0 /*view_id*/,
                       false /*error_on_md_query*/);
   }
 
@@ -404,6 +422,7 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
   void set_router_accepting_ports() {
     router_port_rw = port_pool_.get_next_available();
     router_port_ro = port_pool_.get_next_available();
+    router_port_rw_split = port_pool_.get_next_available();
     router_port_x_rw = port_pool_.get_next_available();
     router_port_x_ro = port_pool_.get_next_available();
   }
@@ -525,9 +544,10 @@ TEST_F(TestRestApiEnable, ensure_rest_is_disabled) {
   EXPECT_FALSE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
        cert_file_t::k_router_cert}));
-  assert_rest_config(config_path, false);
+  ASSERT_NO_FATAL_FAILURE(assert_rest_config(config_path, false));
 
   auto &router = ProcessManager::launch_router({"-c", config_path.str()});
+  router.set_logging_path(temp_test_dir.name() + "/log", "mysqlrouter.log");
 
   EXPECT_EQ(std::error_code{}, router.send_clean_shutdown_event());
   EXPECT_EQ(0, router.wait_for_exit());
@@ -567,11 +587,12 @@ TEST_F(TestRestApiEnable, ensure_rest_works_on_custom_port) {
   EXPECT_TRUE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
        cert_file_t::k_router_cert}));
-  assert_rest_config(config_path, true);
+  ASSERT_NO_FATAL_FAILURE(assert_rest_config(config_path, true));
 
-  ProcessManager::launch_router({"-c", config_path.str()});
+  auto &router = ProcessManager::launch_router({"-c", config_path.str()});
+  router.set_logging_path(temp_test_dir.name() + "/log", "mysqlrouter.log");
 
-  assert_rest_works(custom_port);
+  ASSERT_NO_FATAL_FAILURE(assert_rest_works(custom_port));
 }
 
 class UseEdgeHttpsPortValues : public TestRestApiEnable,
@@ -697,7 +718,7 @@ class RestApiEnableUserCertificates
  * WL13906:TS_FR04_01
  */
 TEST_P(RestApiEnableUserCertificates, ensure_rest_works_with_user_certs) {
-  create_cert_files(GetParam());
+  ASSERT_NO_FATAL_FAILURE(create_cert_files(GetParam()));
   auto &router_bootstrap =
       do_bootstrap({"--https-port", std::to_string(custom_port)});
   const auto expected_message =
@@ -708,12 +729,13 @@ TEST_P(RestApiEnableUserCertificates, ensure_rest_works_with_user_certs) {
 
   EXPECT_TRUE(certificate_files_exists(
       {cert_file_t::k_router_key, cert_file_t::k_router_cert}));
-  assert_rest_config(config_path, true);
+  ASSERT_NO_FATAL_FAILURE(assert_rest_config(config_path, true));
   EXPECT_TRUE(certificate_files_not_changed(GetParam()));
 
-  ProcessManager::launch_router({"-c", config_path.str()});
+  auto &router = ProcessManager::launch_router({"-c", config_path.str()});
+  router.set_logging_path(temp_test_dir.name() + "/log", "mysqlrouter.log");
 
-  assert_rest_works(custom_port);
+  ASSERT_NO_FATAL_FAILURE(assert_rest_works(custom_port));
 }
 
 INSTANTIATE_TEST_SUITE_P(
@@ -854,9 +876,11 @@ TEST_P(RestApiInvalidUserCerts,
             GetParam());
   EXPECT_EQ(get_file_output(datadir_path.join(router_cert_filename).str()),
             GetParam());
-  assert_rest_config(config_path, true);
+  ASSERT_NO_FATAL_FAILURE(assert_rest_config(config_path, true));
 
   auto &router = launch_router({"-c", config_path.str()}, EXIT_FAILURE);
+  router.set_logging_path(temp_test_dir.name() + "/log", "mysqlrouter.log");
+
   check_exit_code(router, EXIT_FAILURE);
 
   const std::string log_error =
@@ -864,9 +888,7 @@ TEST_P(RestApiInvalidUserCerts,
       datadir_path.real_path().join(router_key_filename).str() +
       "' or SSL certificate file '" +
       datadir_path.real_path().join(router_cert_filename).str() + "' failed";
-  EXPECT_THAT(
-      router.get_logfile_content("mysqlrouter.log", get_logging_dir().str()),
-      ::testing::HasSubstr(log_error));
+  EXPECT_THAT(router.get_logfile_content(), ::testing::HasSubstr(log_error));
 }
 
 INSTANTIATE_TEST_SUITE_P(CheckRestApiInvalidUserCerts, RestApiInvalidUserCerts,
@@ -945,12 +967,14 @@ class TestRestApiEnableBootstrapFailover : public TestRestApiEnable {
   }
 
   void setup_mocks(const bool failover_successful) {
+    std::vector<uint16_t> classic_ports;
     for (auto i = 0; i < k_node_count; ++i) {
-      const auto port = port_pool_.get_next_available();
-      gr_nodes.emplace_back(port);
-      cluster_nodes.emplace_back(port);
+      classic_ports.emplace_back(port_pool_.get_next_available());
     }
 
+    gr_nodes = classic_ports_to_gr_nodes(classic_ports);
+    cluster_nodes = classic_ports_to_cluster_nodes(classic_ports);
+
     for (auto i = 0; i < k_node_count; ++i) {
       cluster_http_port = port_pool_.get_next_available();
       auto port = gr_nodes[i].classic_port;
@@ -974,7 +998,7 @@ class TestRestApiEnableBootstrapFailover : public TestRestApiEnable {
                       .wait_for_rest_endpoint_ready());
 
       set_mock_metadata(cluster_http_port, cluster_id, gr_nodes, 0,
-                        cluster_nodes, 0, 0, false, gr_member_ip, "",
+                        cluster_nodes, 0, false, gr_member_ip, "",
                         metadata_version, cluster_name);
     }
 
@@ -1015,11 +1039,12 @@ TEST_F(TestRestApiEnableBootstrapFailover,
   EXPECT_TRUE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
        cert_file_t::k_router_cert}));
-  assert_rest_config(config_path, true);
+  ASSERT_NO_FATAL_FAILURE(assert_rest_config(config_path, true));
 
-  ProcessManager::launch_router({"-c", config_path.str()});
+  auto &router = ProcessManager::launch_router({"-c", config_path.str()});
+  router.set_logging_path(temp_test_dir.name() + "/log", "mysqlrouter.log");
 
-  assert_rest_works(rest_port);
+  ASSERT_NO_FATAL_FAILURE(assert_rest_works(rest_port));
 }
 
 /**
