@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2021, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2021, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -68,6 +69,10 @@ class SocketCloseTest : public RouterComponentTest {
     auto default_section = get_DEFAULT_defaults();
     init_keyring(default_section, get_test_temp_dir_name(), router_user,
                  router_password);
+    const auto state_file =
+        create_state_file(get_test_temp_dir_name(),
+                          create_state_file_content("uuid", "", node_ports, 0));
+    default_section["dynamic_state"] = state_file;
 
     // launch the router
     const std::string conf_file = create_config_file(
@@ -112,24 +117,13 @@ class SocketCloseTest : public RouterComponentTest {
         gr_nodes[0].member_role = "SECONDARY";
         cluster_nodes[0].role = "SECONDARY";
       }
-      set_mock_metadata(node_http_ports[i], "", gr_nodes, i, cluster_nodes, 0,
-                        false, "localhost");
+      set_mock_metadata(node_http_ports[i], "uuid", gr_nodes, i, cluster_nodes,
+                        0, false, "localhost");
     }
   }
 
   std::string get_metadata_cache_section(
-      std::vector<uint16_t> metadata_server_ports,
       ClusterType cluster_type = ClusterType::GR_V2) {
-    std::string bootstrap_server_addresses;
-    bool use_comma = false;
-    for (const auto &port : metadata_server_ports) {
-      if (use_comma) {
-        bootstrap_server_addresses += ",";
-      } else {
-        use_comma = true;
-      }
-      bootstrap_server_addresses += "mysql://localhost:" + std::to_string(port);
-    }
     const std::string cluster_type_str =
         (cluster_type == ClusterType::RS_V2) ? "rs" : "gr";
 
@@ -138,8 +132,8 @@ class SocketCloseTest : public RouterComponentTest {
            cluster_type_str +
            "\n"
            "router_id=1\n"
-           "bootstrap_server_addresses=" +
-           bootstrap_server_addresses + "\n" + "user=" + router_user +
+           "user=" +
+           router_user +
            "\n"
            "connect_timeout=1\n"
            "metadata_cluster=test\n" +
@@ -150,8 +144,7 @@ class SocketCloseTest : public RouterComponentTest {
   std::string get_metadata_cache_routing_section(
       const std::optional<uint16_t> &router_port,
       const std::optional<std::string> &router_socket, const std::string &role,
-      const std::string &strategy, const std::string &mode = "",
-      const std::string &section_name = "default",
+      const std::string &strategy, const std::string &section_name = "default",
       const std::string &protocol = "classic") const {
     std::vector<std::pair<std::string, std::string>> options{
         {"destinations", "metadata-cache://test/default?role=" + role},
@@ -162,7 +155,6 @@ class SocketCloseTest : public RouterComponentTest {
     if (router_socket) options.emplace_back("socket", *router_socket);
 
     if (!strategy.empty()) options.emplace_back("routing_strategy", strategy);
-    if (!mode.empty()) options.emplace_back("mode", mode);
 
     return mysql_harness::ConfigBuilder::build_section(
         "routing:" + section_name, options);
@@ -197,7 +189,7 @@ class SocketCloseTest : public RouterComponentTest {
   void setup_router(ClusterType cluster_type, const Acceptors acceptors,
                     const bool read_only = false) {
     const std::string metadata_cache_section =
-        get_metadata_cache_section(node_ports, cluster_type);
+        get_metadata_cache_section(cluster_type);
 
     const bool use_tcp_port_acceptors =
         acceptors & static_cast<Acceptors>(AcceptorType::TcpSocket);
@@ -208,15 +200,15 @@ class SocketCloseTest : public RouterComponentTest {
     std::string routing_rw_section{""};
     if (!read_only) {
       routing_rw_section = get_metadata_cache_routing_section(
-          router_rw_port, router_rw_socket, "PRIMARY", "round-robin", "", "rw");
+          router_rw_port, router_rw_socket, "PRIMARY", "round-robin", "rw");
       routing_rw_section += get_metadata_cache_routing_section(
-          router_rw_x_port, router_rw_x_socket, "PRIMARY", "round-robin", "",
+          router_rw_x_port, router_rw_x_socket, "PRIMARY", "round-robin",
           "x_rw", "x");
     }
     std::string routing_ro_section = get_metadata_cache_routing_section(
-        router_ro_port, router_ro_socket, "SECONDARY", "round-robin", "", "ro");
+        router_ro_port, router_ro_socket, "SECONDARY", "round-robin", "ro");
     routing_ro_section += get_metadata_cache_routing_section(
-        router_ro_x_port, router_ro_x_socket, "SECONDARY", "round-robin", "",
+        router_ro_x_port, router_ro_x_socket, "SECONDARY", "round-robin",
         "x_ro", "x");
 
     router =
@@ -238,7 +230,7 @@ class SocketCloseTest : public RouterComponentTest {
   void toggle_auth_failure(const bool toggle, const uint16_t http_port,
                            const std::vector<uint16_t> &nodes) {
     auto globals =
-        mock_GR_metadata_as_json("", classic_ports_to_gr_nodes(nodes), 0,
+        mock_GR_metadata_as_json("uuid", classic_ports_to_gr_nodes(nodes), 0,
                                  classic_ports_to_cluster_nodes(nodes));
     JsonAllocator allocator;
 
@@ -283,8 +275,8 @@ class SocketCloseTest : public RouterComponentTest {
   void simulate_cluster_node_down(const std::vector<uint16_t> &node_ports,
                                   const uint16_t http_port) {
     auto globals =
-        mock_GR_metadata_as_json("", classic_ports_to_gr_nodes(node_ports), 0,
-                                 classic_ports_to_cluster_nodes(node_ports));
+        mock_GR_metadata_as_json("uuid", classic_ports_to_gr_nodes(node_ports),
+                                 0, classic_ports_to_cluster_nodes(node_ports));
     JsonAllocator allocator;
     globals.AddMember("transaction_count", 0, allocator);
     // Empty, node is not taken into account
@@ -304,7 +296,8 @@ class SocketCloseTest : public RouterComponentTest {
       gr_nodes[0].member_role = "SECONDARY";
       cluster_nodes[0].role = "SECONDARY";
     }
-    auto globals = mock_GR_metadata_as_json("", gr_nodes, 0, cluster_nodes);
+
+    auto globals = mock_GR_metadata_as_json("uuid", gr_nodes, 0, cluster_nodes);
     JsonAllocator allocator;
     globals.AddMember("transaction_count", 0, allocator);
     globals.AddMember("cluster_type",
@@ -598,10 +591,11 @@ TEST_P(SocketCloseOnMetadataUnavailable, 1RW2RO) {
 #endif
 
   SCOPED_TRACE("// launch cluster with 3 nodes, 1 RW/2 RO");
-  setup_cluster(3, GetParam().tracefile);
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(3, GetParam().tracefile));
 
   SCOPED_TRACE("// launch the router with metadata-cache configuration");
-  setup_router(GetParam().cluster_type, GetParam().acceptors);
+  ASSERT_NO_FATAL_FAILURE(
+      setup_router(GetParam().cluster_type, GetParam().acceptors));
   SCOPED_TRACE("// check if both RO and RW ports are used");
   if (use_tcp_port_acceptors) {
     for (const auto &port :
@@ -749,10 +743,11 @@ TEST_P(SocketCloseOnMetadataUnavailable, 1RW) {
 #endif
 
   SCOPED_TRACE("// launch cluster with only RW node");
-  setup_cluster(1, GetParam().tracefile);
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(1, GetParam().tracefile));
 
   SCOPED_TRACE("// launch the router with metadata-cache configuration");
-  setup_router(GetParam().cluster_type, GetParam().acceptors);
+  ASSERT_NO_FATAL_FAILURE(
+      setup_router(GetParam().cluster_type, GetParam().acceptors));
 
   SCOPED_TRACE("// check if RW port is used");
   if (use_tcp_port_acceptors) {
@@ -836,11 +831,13 @@ TEST_P(SocketCloseOnMetadataUnavailable, 1RO) {
 #endif
 
   SCOPED_TRACE("// launch cluster with only RO node");
-  setup_cluster(1, GetParam().tracefile, /*no_primary*/ true);
+  ASSERT_NO_FATAL_FAILURE(
+      setup_cluster(1, GetParam().tracefile, /*no_primary*/ true));
 
   SCOPED_TRACE("// launch the router with metadata-cache configuration");
-  setup_router(GetParam().cluster_type, GetParam().acceptors,
-               /*read_only*/ true);
+  ASSERT_NO_FATAL_FAILURE(setup_router(GetParam().cluster_type,
+                                       GetParam().acceptors,
+                                       /*read_only*/ true));
 
   SCOPED_TRACE("// check if RO port is used");
   if (use_tcp_port_acceptors) {
@@ -924,11 +921,13 @@ TEST_P(SocketCloseOnMetadataUnavailable, 2RO) {
 #endif
 
   SCOPED_TRACE("// launch cluster with 2 RO nodes");
-  setup_cluster(2, GetParam().tracefile, /*no_primary*/ true);
+  ASSERT_NO_FATAL_FAILURE(
+      setup_cluster(2, GetParam().tracefile, /*no_primary*/ true));
 
   SCOPED_TRACE("// launch the router with metadata-cache configuration");
-  setup_router(GetParam().cluster_type, GetParam().acceptors,
-               /*read_only*/ true);
+  ASSERT_NO_FATAL_FAILURE(setup_router(GetParam().cluster_type,
+                                       GetParam().acceptors,
+                                       /*read_only*/ true));
 
   SCOPED_TRACE("// check if RO port is used");
   if (use_tcp_port_acceptors) {
@@ -1067,13 +1066,81 @@ INSTANTIATE_TEST_SUITE_P(
                 Acceptors(AcceptorType::UnixSocket))),
     get_test_description);
 
+class ErrmsgResponderBase {
+ public:
+  // error-code to return on connect
+  static constexpr const uint16_t error_code{1130};
+  // error-msg to return on connect
+  static constexpr const char error_msg[] = "You shall not pass";
+};
+
+template <class Sock>
+class ErrmsgResponder : public ErrmsgResponderBase {
+ public:
+  using socket_type = Sock;
+
+  explicit ErrmsgResponder(socket_type sock) : sock_(std::move(sock)) {}
+
+  stdx::expected<void, std::error_code> respond() {
+    std::vector<uint8_t> err_frame;
+
+    const auto encode_res =
+        classic_protocol::encode<classic_protocol::frame::Frame<
+            classic_protocol::message::server::Error>>(
+            {0, {error_code, error_msg, "HY000"}}, {},
+            net::dynamic_buffer(err_frame));
+    if (!encode_res) return stdx::unexpected(encode_res.error());
+
+    const auto write_res = net::write(sock_, net::buffer(err_frame));
+    if (!write_res) return stdx::unexpected(write_res.error());
+
+    // wait until the client closed the connection on us.
+    //
+    while (true) {
+      std::vector<std::string> drainer;
+      const auto read_res = net::read(sock_, net::dynamic_buffer(drainer));
+
+      if (!read_res &&
+          read_res.error() == make_error_code(net::stream_errc::eof)) {
+        break;
+      }
+
+      // looks like something else happened. At least log it.
+      if (read_res) {
+        std::cerr << __LINE__ << ": " << read_res.value() << std::endl;
+      } else {
+        return stdx::unexpected(read_res.error());
+      }
+    }
+
+    return {};
+  }
+
+ private:
+  socket_type sock_;
+};
+
 template <class AcceptorType>
 class AcceptingEndpointUser {
  public:
-  // error-code to return on connect
-  static const uint16_t error_code{1130};
-  // error-msg to return on connect
-  static const char error_msg[];
+  class AcceptCompletor {
+   public:
+    AcceptCompletor(AcceptorType &acceptor) : acceptor_(acceptor) {}
+
+    void operator()(std::error_code ec, auto client_sock) {
+      if (ec == std::errc::operation_canceled) return;
+
+      ErrmsgResponder responder(std::move(client_sock));
+
+      responder.respond();
+
+      // accept the next one.
+      acceptor_.async_accept(AcceptCompletor(acceptor_));
+    }
+
+   private:
+    AcceptorType &acceptor_;
+  };
 
   virtual ~AcceptingEndpointUser() { unlock(); }
 
@@ -1090,11 +1157,13 @@ class AcceptingEndpointUser {
   }
 
   virtual void unlock() {
-    acceptor_.close();
+    acceptor_.close();  // stops the io-ctx too as there is no other user.
+
     if (worker_.joinable()) worker_.join();
 
     if (worker_ec_) {
-      FAIL() << "acceptor() failed after accept() with: " << worker_ec_;
+      FAIL() << "acceptor() failed after accept() with: " << worker_ec_ << " "
+             << worker_ec_.message();
     }
   }
 
@@ -1104,62 +1173,16 @@ class AcceptingEndpointUser {
   virtual bool try_lock() {
     if (!open_and_bind()) return false;
 
-    const auto &listen_res = acceptor_.listen(128);
+    const auto listen_res = acceptor_.listen(128);
     if (!listen_res) {
       return false;
     }
 
     // spawn off a thread to handle a connect.
     worker_ = std::thread([this]() {
-      acceptor_.async_accept([this](std::error_code ec, auto client_sock) {
-        if (ec == std::errc::operation_canceled) return;
+      acceptor_.async_accept(AcceptCompletor(acceptor_));
 
-        std::vector<uint8_t> err_frame;
-
-        const auto encode_res =
-            classic_protocol::encode<classic_protocol::frame::Frame<
-                classic_protocol::message::server::Error>>(
-                {0, {error_code, error_msg, "HY000"}}, {},
-                net::dynamic_buffer(err_frame));
-        if (!encode_res) {
-          worker_ec_ = encode_res.error();
-          return;
-        }
-
-        // using the full type as sun-cc doesn't like 'auto' here and gives:
-        //
-        // The operation "! ?" is illegal.
-        const stdx::expected<size_t, std::error_code> write_res =
-            net::write(client_sock, net::buffer(err_frame));
-        if (!write_res) {
-          worker_ec_ = write_res.error();
-          return;
-        }
-
-        // wait until the client closed the connection on us.
-        //
-        while (true) {
-          std::vector<std::string> drainer;
-          const auto read_res =
-              net::read(client_sock, net::dynamic_buffer(drainer));
-
-          if (!read_res &&
-              read_res.error() == make_error_code(net::stream_errc::eof)) {
-            break;
-          }
-
-          // looks like something else happened. At least log it.
-          if (read_res) {
-            std::cerr << __LINE__ << ": " << read_res.value() << std::endl;
-          } else {
-            worker_ec_ = read_res.error();
-            return;
-          }
-        }
-      });
-
-      // accept zero-or-one connection.
-      io_ctx_.run_one();
+      io_ctx_.run();
     });
 
     return true;
@@ -1172,12 +1195,6 @@ class AcceptingEndpointUser {
   AcceptorType acceptor_{io_ctx_};
 };
 
-template <class T>
-const uint16_t AcceptingEndpointUser<T>::error_code;
-
-template <class T>
-const char AcceptingEndpointUser<T>::error_msg[] = "You shall not pass";
-
 class TCPPortUser : public AcceptingEndpointUser<net::ip::tcp::acceptor> {
  public:
   TCPPortUser(std::string hostname, const uint16_t port)
@@ -1223,8 +1240,6 @@ class UnixSocketUser
 
  protected:
   bool open_and_bind() override {
-    acceptor_.set_option(net::socket_base::reuse_address{true});
-
     const auto open_res = acceptor_.open();
     if (!open_res) {
       return false;
@@ -1244,7 +1259,7 @@ class UnixSocketUser
 
 TEST_F(SocketCloseTest, StaticRoundRobinTCPPort) {
   SCOPED_TRACE("// launch cluster with one node");
-  setup_cluster(1, "my_port.js");
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(1, "my_port.js"));
 
   router_rw_port = port_pool_.get_next_available();
   const auto router_rw_port_str = std::to_string(*router_rw_port);
@@ -1287,7 +1302,7 @@ TEST_F(SocketCloseTest, StaticRoundRobinTCPPort) {
       json_metadata, node_ports[0], EXIT_SUCCESS, false, node_http_ports[0]));
 
   set_mock_metadata(
-      node_http_ports[0], "", classic_ports_to_gr_nodes(node_ports), 0,
+      node_http_ports[0], "uuid", classic_ports_to_gr_nodes(node_ports), 0,
       classic_ports_to_cluster_nodes(node_ports), 0, false, "localhost");
 
   SCOPED_TRACE("// check we can connect to tcp:" + router_rw_port_str +
@@ -1297,8 +1312,8 @@ TEST_F(SocketCloseTest, StaticRoundRobinTCPPort) {
     try_connection("127.0.0.1", *router_rw_port, custom_user, custom_password);
     FAIL() << "should have failed";
   } catch (const MySQLSession::Error &e) {
-    EXPECT_EQ(e.code(), TCPPortUser::error_code);
-    EXPECT_THAT(e.what(), ::testing::HasSubstr(TCPPortUser::error_msg));
+    EXPECT_EQ(e.code(), ErrmsgResponderBase::error_code);
+    EXPECT_THAT(e.what(), ::testing::HasSubstr(ErrmsgResponderBase::error_msg));
   }
 
   // sleep for a while to test that when the quarantine wants to reopen the
@@ -1321,9 +1336,10 @@ TEST_F(SocketCloseTest, StaticRoundRobinTCPPort) {
 }
 
 #ifndef _WIN32
+
 TEST_F(SocketCloseTest, StaticRoundRobinUnixSocket) {
   SCOPED_TRACE("// launch cluster with one node");
-  setup_cluster(1, "my_port.js");
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(1, "my_port.js"));
 
   router_rw_socket = get_test_temp_dir_name() + "/mysql.socket";
 
@@ -1344,8 +1360,15 @@ TEST_F(SocketCloseTest, StaticRoundRobinUnixSocket) {
   EXPECT_NO_THROW(cluster_nodes[0]->send_clean_shutdown_event());
   EXPECT_NO_THROW(cluster_nodes[0]->wait_for_exit());
 
-  EXPECT_THROW(try_connection(*router_rw_socket, custom_user, custom_password),
-               std::runtime_error);
+  try {
+    try_connection(*router_rw_socket, custom_user, custom_password);
+    FAIL() << "expected to fail";
+  } catch (const MySQLSession::Error &e) {
+    // /tmp/router-0tsoKZ/mysql.socket: Can't connect to remote MySQL server
+    // (2003)
+    EXPECT_EQ(e.code(), 2003) << e.what();
+  }
+
   EXPECT_FALSE(wait_file_exists(*router_rw_socket, false, 10s));
 
   SCOPED_TRACE("// block router from binding to unix socket:" +
@@ -1365,7 +1388,7 @@ TEST_F(SocketCloseTest, StaticRoundRobinUnixSocket) {
       json_metadata, node_ports[0], EXIT_SUCCESS, false, node_http_ports[0]));
 
   set_mock_metadata(
-      node_http_ports[0], "", classic_ports_to_gr_nodes(node_ports), 0,
+      node_http_ports[0], "uuid", classic_ports_to_gr_nodes(node_ports), 0,
       classic_ports_to_cluster_nodes(node_ports), 0, false, "localhost");
 
   SCOPED_TRACE("// check we can connect to unix socket:" + *router_rw_socket +
@@ -1375,8 +1398,8 @@ TEST_F(SocketCloseTest, StaticRoundRobinUnixSocket) {
     try_connection(*router_rw_socket, custom_user, custom_password);
     FAIL() << "should have failed";
   } catch (const MySQLSession::Error &e) {
-    EXPECT_EQ(e.code(), UnixSocketUser::error_code);
-    EXPECT_THAT(e.what(), ::testing::HasSubstr(UnixSocketUser::error_msg));
+    EXPECT_EQ(e.code(), ErrmsgResponderBase::error_code);
+    EXPECT_THAT(e.what(), ::testing::HasSubstr(ErrmsgResponderBase::error_msg));
   }
 
   // sleep for a while to test that when the quarantine wants to reopen the
@@ -1442,18 +1465,19 @@ class FailToOpenROSocketAfterStartup
 
 TEST_P(FailToOpenROSocketAfterStartup, ROportTaken) {
   SCOPED_TRACE("// launch cluster with 3 nodes, 1 RW/2 RO");
-  setup_cluster(3, GetParam().tracefile);
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(3, GetParam().tracefile));
   const auto test_port = port_mapping.at(GetParam().unavailable_ports[0]);
 
   SCOPED_TRACE("// launch the router with metadata-cache configuration");
-  setup_router(GetParam().cluster_type, Acceptors(AcceptorType::TcpSocket));
+  ASSERT_NO_FATAL_FAILURE(setup_router(GetParam().cluster_type,
+                                       Acceptors(AcceptorType::TcpSocket)));
   EXPECT_TRUE(wait_for_port_used(*router_ro_port));
 
   SCOPED_TRACE("// RO nodes hidden");
   auto cluster_nodes = classic_ports_to_cluster_nodes(node_ports);
   cluster_nodes[1].attributes = R"({"tags" : {"_hidden": true} })";
   cluster_nodes[2].attributes = R"({"tags" : {"_hidden": true} })";
-  set_mock_metadata(node_http_ports[0], "",
+  set_mock_metadata(node_http_ports[0], "uuid",
                     classic_ports_to_gr_nodes(node_ports), 0, cluster_nodes, 0,
                     false, "127.0.0.1");
 
@@ -1469,7 +1493,7 @@ TEST_P(FailToOpenROSocketAfterStartup, ROportTaken) {
 
   SCOPED_TRACE("// Unhide one RO node");
   cluster_nodes[2].attributes = "";
-  set_mock_metadata(node_http_ports[0], "",
+  set_mock_metadata(node_http_ports[0], "uuid",
                     classic_ports_to_gr_nodes(node_ports), 0, cluster_nodes, 0,
                     false, "127.0.0.1");
   EXPECT_TRUE(wait_for_transaction_count_increase(node_http_ports[0], 4));
@@ -1518,17 +1542,18 @@ class FailToOpenRWSocketAfterStartup
 
 TEST_P(FailToOpenRWSocketAfterStartup, RWportTaken) {
   SCOPED_TRACE("// launch cluster with 3 nodes, 1 RW/2 RO");
-  setup_cluster(3, GetParam().tracefile);
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(3, GetParam().tracefile));
   const auto test_port = port_mapping.at(GetParam().unavailable_ports[0]);
 
   SCOPED_TRACE("// launch the router with metadata-cache configuration");
-  setup_router(GetParam().cluster_type, Acceptors(AcceptorType::TcpSocket));
+  ASSERT_NO_FATAL_FAILURE(setup_router(GetParam().cluster_type,
+                                       Acceptors(AcceptorType::TcpSocket)));
   EXPECT_TRUE(wait_for_port_used(*router_rw_port));
 
   SCOPED_TRACE("// RW node hidden");
   auto cluster_nodes = classic_ports_to_cluster_nodes(node_ports);
   cluster_nodes[0].attributes = R"({"tags" : {"_hidden": true} })";
-  set_mock_metadata(node_http_ports[0], "",
+  set_mock_metadata(node_http_ports[0], "uuid",
                     classic_ports_to_gr_nodes(node_ports), 0, cluster_nodes, 0,
                     false, "127.0.0.1");
   EXPECT_TRUE(wait_for_transaction_count_increase(node_http_ports[0], 4));
@@ -1543,7 +1568,7 @@ TEST_P(FailToOpenRWSocketAfterStartup, RWportTaken) {
 
   SCOPED_TRACE("// Unhide RW node");
   cluster_nodes[0].attributes = "";
-  set_mock_metadata(node_http_ports[0], "",
+  set_mock_metadata(node_http_ports[0], "uuid",
                     classic_ports_to_gr_nodes(node_ports), 0, cluster_nodes, 0,
                     false, "127.0.0.1");
   EXPECT_TRUE(wait_for_transaction_count_increase(node_http_ports[0], 4));
@@ -1592,7 +1617,7 @@ class FailToOpenSocketOnStartup
 
 TEST_P(FailToOpenSocketOnStartup, FailOnStartup) {
   SCOPED_TRACE("// launch cluster with 1RW/2RO nodes");
-  setup_cluster(3, GetParam().tracefile);
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(3, GetParam().tracefile));
 
   SCOPED_TRACE("// bind sockets");
   std::vector<std::unique_ptr<TCPPortUser>> socket_users;
@@ -1607,15 +1632,15 @@ TEST_P(FailToOpenSocketOnStartup, FailOnStartup) {
 
   SCOPED_TRACE("// start router against sockets that are in use.");
   const std::string metadata_cache_section =
-      get_metadata_cache_section(node_ports, GetParam().cluster_type);
+      get_metadata_cache_section(GetParam().cluster_type);
   std::string routing_section = get_metadata_cache_routing_section(
-      router_rw_port, "PRIMARY", "round-robin", "", "rw");
+      router_rw_port, "PRIMARY", "round-robin", "rw");
   routing_section += get_metadata_cache_routing_section(
-      router_rw_x_port, "PRIMARY", "round-robin", "", "x_rw", "x");
+      router_rw_x_port, "PRIMARY", "round-robin", "x_rw", "x");
   routing_section += get_metadata_cache_routing_section(
-      router_ro_port, "SECONDARY", "round-robin", "", "ro");
+      router_ro_port, "SECONDARY", "round-robin", "ro");
   routing_section += get_metadata_cache_routing_section(
-      router_ro_x_port, "SECONDARY", "round-robin", "", "x_ro", "x");
+      router_ro_x_port, "SECONDARY", "round-robin", "x_ro", "x");
 
   auto &router =
       launch_router(metadata_cache_section, routing_section, EXIT_FAILURE,
@@ -1679,19 +1704,19 @@ class RoundRobinFallback
 TEST_P(RoundRobinFallback, RoundRobinFallbackTest) {
   const size_t NUM_NODES = 3;
   SCOPED_TRACE("// launch cluster with 1RW/2RO nodes");
-  setup_cluster(NUM_NODES, GetParam().tracefile);
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(NUM_NODES, GetParam().tracefile));
 
   router_rw_port = port_pool_.get_next_available();
   router_ro_port = port_pool_.get_next_available();
 
   SCOPED_TRACE("// launch the router with metadata-cache configuration");
   const std::string metadata_cache_section =
-      get_metadata_cache_section(node_ports, GetParam().cluster_type);
+      get_metadata_cache_section(GetParam().cluster_type);
   std::string routing_section = get_metadata_cache_routing_section(
-      router_rw_port, std::nullopt, "PRIMARY", "round-robin", "", "rw");
+      router_rw_port, std::nullopt, "PRIMARY", "round-robin", "rw");
   routing_section += get_metadata_cache_routing_section(
       router_ro_port, std::nullopt, "SECONDARY", "round-robin-with-fallback",
-      "", "ro");
+      "ro");
 
   launch_router(metadata_cache_section, routing_section, EXIT_SUCCESS,
                 /*wait_for_notify_ready=*/30s);
@@ -1703,7 +1728,7 @@ TEST_P(RoundRobinFallback, RoundRobinFallbackTest) {
   auto cluster_nodes = classic_ports_to_cluster_nodes(node_ports);
   cluster_nodes[1].attributes = R"({"tags" : {"_hidden": true} })";
   cluster_nodes[2].attributes = R"({"tags" : {"_hidden": true} })";
-  set_mock_metadata(node_http_ports[0], "",
+  set_mock_metadata(node_http_ports[0], "uuid",
                     classic_ports_to_gr_nodes(node_ports), 0, cluster_nodes, 0,
                     false, "127.0.0.1");
   EXPECT_TRUE(wait_for_transaction_count_increase(node_http_ports[0], 4));
@@ -1719,7 +1744,7 @@ TEST_P(RoundRobinFallback, RoundRobinFallbackTest) {
   SCOPED_TRACE("// Unhide RO nodes");
   cluster_nodes[1].attributes = "";
   cluster_nodes[2].attributes = "";
-  set_mock_metadata(node_http_ports[0], "",
+  set_mock_metadata(node_http_ports[0], "uuid",
                     classic_ports_to_gr_nodes(node_ports), 0, cluster_nodes, 0,
                     false, "127.0.0.1");
   ASSERT_NO_FATAL_FAILURE(try_connection("127.0.0.1", *router_ro_port,
@@ -1729,7 +1754,7 @@ TEST_P(RoundRobinFallback, RoundRobinFallbackTest) {
 
   SCOPED_TRACE("// Hide primary node");
   cluster_nodes[0].attributes = R"({"tags" : {"_hidden": true} })";
-  set_mock_metadata(node_http_ports[0], "",
+  set_mock_metadata(node_http_ports[0], "uuid",
                     classic_ports_to_gr_nodes(node_ports), 0, cluster_nodes, 0,
                     false, "127.0.0.1");
   EXPECT_TRUE(wait_for_transaction_count_increase(node_http_ports[0], 4));
@@ -1757,18 +1782,18 @@ class FirstAvailableDestMetadataCache
 TEST_P(FirstAvailableDestMetadataCache, FirstAvailableDestMetadataCacheTest) {
   const size_t NUM_NODES = 3;
   SCOPED_TRACE("// launch cluster with 1RW/2RO nodes");
-  setup_cluster(NUM_NODES, GetParam().tracefile);
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(NUM_NODES, GetParam().tracefile));
 
   router_rw_port = port_pool_.get_next_available();
   router_ro_port = port_pool_.get_next_available();
 
   SCOPED_TRACE("// launch the router with metadata-cache configuration");
   const std::string metadata_cache_section =
-      get_metadata_cache_section(node_ports, GetParam().cluster_type);
+      get_metadata_cache_section(GetParam().cluster_type);
   std::string routing_section = get_metadata_cache_routing_section(
-      router_rw_port, std::nullopt, "PRIMARY", "first-available", "", "rw");
+      router_rw_port, std::nullopt, "PRIMARY", "first-available", "rw");
   routing_section += get_metadata_cache_routing_section(
-      router_ro_port, std::nullopt, "SECONDARY", "first-available", "", "ro");
+      router_ro_port, std::nullopt, "SECONDARY", "first-available", "ro");
 
   launch_router(metadata_cache_section, routing_section, EXIT_SUCCESS,
                 /*wait_for_notify_ready=*/30s);
@@ -1778,7 +1803,7 @@ TEST_P(FirstAvailableDestMetadataCache, FirstAvailableDestMetadataCacheTest) {
 
   SCOPED_TRACE("// Disable both secondary nodes");
   set_mock_metadata(
-      node_http_ports[0], "", classic_ports_to_gr_nodes({node_ports[0]}), 0,
+      node_http_ports[0], "uuid", classic_ports_to_gr_nodes({node_ports[0]}), 0,
       classic_ports_to_cluster_nodes({node_ports[0]}), 0, false, "localhost");
   EXPECT_TRUE(wait_for_transaction_count_increase(node_http_ports[0], 4));
 
@@ -1793,7 +1818,7 @@ TEST_P(FirstAvailableDestMetadataCache, FirstAvailableDestMetadataCacheTest) {
 
   SCOPED_TRACE("// Bring back first RO node");
   set_mock_metadata(
-      node_http_ports[0], "",
+      node_http_ports[0], "uuid",
       classic_ports_to_gr_nodes({node_ports[0], node_ports[1]}), 0,
       classic_ports_to_cluster_nodes({node_ports[0], node_ports[1]}), 0, false,
       "localhost");
@@ -1803,7 +1828,7 @@ TEST_P(FirstAvailableDestMetadataCache, FirstAvailableDestMetadataCacheTest) {
 
   SCOPED_TRACE("// Disable first RO node");
   set_mock_metadata(
-      node_http_ports[0], "", classic_ports_to_gr_nodes({node_ports[0]}), 0,
+      node_http_ports[0], "uuid", classic_ports_to_gr_nodes({node_ports[0]}), 0,
       classic_ports_to_cluster_nodes({node_ports[0]}), 0, false, "localhost");
   EXPECT_TRUE(wait_for_transaction_count_increase(node_http_ports[0], 4));
   EXPECT_TRUE(wait_for_port_used(*router_rw_port));
@@ -1811,7 +1836,7 @@ TEST_P(FirstAvailableDestMetadataCache, FirstAvailableDestMetadataCacheTest) {
 
   SCOPED_TRACE("// Bring back second RO node");
   set_mock_metadata(
-      node_http_ports[0], "",
+      node_http_ports[0], "uuid",
       classic_ports_to_gr_nodes({node_ports[0], node_ports[2]}), 0,
       classic_ports_to_cluster_nodes({node_ports[0], node_ports[2]}), 0, false,
       "localhost");
@@ -1821,7 +1846,7 @@ TEST_P(FirstAvailableDestMetadataCache, FirstAvailableDestMetadataCacheTest) {
 
   SCOPED_TRACE("// Disable first RO node");
   set_mock_metadata(
-      node_http_ports[0], "", classic_ports_to_gr_nodes({node_ports[0]}), 0,
+      node_http_ports[0], "uuid", classic_ports_to_gr_nodes({node_ports[0]}), 0,
       classic_ports_to_cluster_nodes({node_ports[0]}), 0, false, "localhost");
   EXPECT_TRUE(wait_for_transaction_count_increase(node_http_ports[0], 4));
   EXPECT_TRUE(wait_for_port_used(*router_rw_port));
@@ -1906,7 +1931,7 @@ class SharedQuarantineSocketClose
       public ::testing::WithParamInterface<SharedQuarantineSocketCloseParam> {};
 
 TEST_P(SharedQuarantineSocketClose, cross_plugin_socket_shutdown) {
-  setup_cluster(1, "metadata_dynamic_nodes_v2_gr.js");
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(1, "metadata_dynamic_nodes_v2_gr.js"));
   const auto bind_port_r1 = port_pool_.get_next_available();
   const auto bind_port_r2 = port_pool_.get_next_available();
   const std::string routing_section{
@@ -1969,11 +1994,11 @@ TEST_F(SharedQuarantineSocketCloseWithFallback,
   const auto bind_port_r2 = port_pool_.get_next_available();
   const auto bind_port_r3 = port_pool_.get_next_available();
   const std::string metadata_cache_section =
-      get_metadata_cache_section(node_ports, ClusterType::GR_V2);
+      get_metadata_cache_section(ClusterType::GR_V2);
   std::string routing_section = get_metadata_cache_routing_section(
-      bind_port_r1, std::nullopt, "PRIMARY", "round-robin", "", "r1");
+      bind_port_r1, std::nullopt, "PRIMARY", "round-robin", "r1");
   routing_section += get_metadata_cache_routing_section(
-      bind_port_r2, std::nullopt, "SECONDARY", "round-robin-with-fallback", "",
+      bind_port_r2, std::nullopt, "SECONDARY", "round-robin-with-fallback",
       "r2");
   routing_section += get_static_routing_section(bind_port_r3, std::nullopt,
                                                 {node_ports[1]}, "round-robin");
