@@ -39,6 +39,7 @@
 #include "sql/sql_class.h"  // Query_arena
 #include "sql/sql_error.h"
 #include "sql/sql_list.h"
+#include "sql/statement/statement_runnable.h"
 
 class Item;
 class Item_param;
@@ -113,6 +114,9 @@ class Reprepare_observer final {
   static constexpr int MAX_REPREPARE_ATTEMPTS = 3;
 };
 
+void rewrite_query(THD *thd);
+void log_execute_line(THD *thd);
+
 bool ask_to_reprepare(THD *thd);
 bool mysql_stmt_precheck(THD *thd, const COM_DATA *com_data,
                          enum enum_server_command cmd,
@@ -131,19 +135,7 @@ void mysql_stmt_get_longdata(THD *thd, Prepared_statement *stmt,
                              uint param_number, uchar *longdata, ulong length);
 bool select_like_stmt_cmd_test(THD *thd, class Sql_cmd_dml *cmd,
                                ulong setup_tables_done_option);
-
-/**
-  Execute a fragment of server code in an isolated context, so that
-  it doesn't leave any effect on THD. THD must have no open tables.
-  The code must not leave any open tables around.
-  The result of execution (if any) is stored in Ed_result.
-*/
-
-class Server_runnable {
- public:
-  virtual bool execute_server_code(THD *thd) = 0;
-  virtual ~Server_runnable();
-};
+void reset_stmt_parameters(Prepared_statement *stmt);
 
 /**
   Execute direct interface.
@@ -151,192 +143,6 @@ class Server_runnable {
   @todo Implement support for prelocked mode.
 */
 
-class Ed_row;
-
-/**
-  Ed_result_set -- a container with result set rows.
-  @todo Implement support for result set metadata and
-  automatic type conversion.
-*/
-
-class Ed_result_set final {
- public:
-  operator List<Ed_row> &() { return *m_rows; }
-  unsigned int size() const { return m_rows->elements; }
-  Ed_row *get_fields() { return m_fields; }
-
-  Ed_result_set(List<Ed_row> *rows_arg, Ed_row *fields, size_t column_count,
-                MEM_ROOT *mem_root_arg);
-
-  /** We don't call member destructors, they all are POD types. */
-  ~Ed_result_set() = default;
-
-  size_t get_field_count() const { return m_column_count; }
-
-  static void *operator new(size_t size, MEM_ROOT *mem_root,
-                            const std::nothrow_t & = std::nothrow) noexcept {
-    return mem_root->Alloc(size);
-  }
-
-  static void operator delete(void *, size_t) noexcept {
-    // Does nothing because m_mem_root is deallocated in the destructor
-  }
-
-  static void operator delete(
-      void *, MEM_ROOT *, const std::nothrow_t &) noexcept { /* never called */
-  }
-
- private:
-  Ed_result_set(const Ed_result_set &);      /* not implemented */
-  Ed_result_set &operator=(Ed_result_set &); /* not implemented */
- private:
-  MEM_ROOT m_mem_root;
-  size_t m_column_count;
-  List<Ed_row> *m_rows;
-  Ed_row *m_fields;
-  Ed_result_set *m_next_rset;
-  friend class Ed_connection;
-};
-
-class Ed_connection final {
- public:
-  /**
-    Construct a new "execute direct" connection.
-
-    The connection can be used to execute SQL statements.
-    If the connection failed to initialize, the error
-    will be returned on the attempt to execute a statement.
-
-    @pre thd  must have no open tables
-              while the connection is used. However,
-              Ed_connection works okay in LOCK TABLES mode.
-              Other properties of THD, such as the current warning
-              information, errors, etc. do not matter and are
-              preserved by Ed_connection. One thread may have many
-              Ed_connections created for it.
-  */
-  Ed_connection(THD *thd);
-
-  /**
-    Execute one SQL statement.
-
-    Until this method is executed, no other methods of
-    Ed_connection can be used. Life cycle of Ed_connection is:
-
-    Initialized -> a statement has been executed ->
-    look at result, move to next result ->
-    look at result, move to next result ->
-    ...
-    moved beyond the last result == Initialized.
-
-    This method can be called repeatedly. Once it's invoked,
-    results of the previous execution are lost.
-
-    A result of execute_direct() can be either:
-
-    - success, no result set rows. In this case get_field_count()
-    returns 0. This happens after execution of INSERT, UPDATE,
-    DELETE, DROP and similar statements. Some other methods, such
-    as get_affected_rows() can be used to retrieve additional
-    result information.
-
-    - success, there are some result set rows (maybe 0). E.g.
-    happens after SELECT. In this case get_field_count() returns
-    the number of columns in a result set and store_result()
-    can be used to retrieve a result set..
-
-    - an error, methods to retrieve error information can
-    be used.
-
-    @return execution status
-    @retval false  success, use get_field_count()
-                   to determine what to do next.
-    @retval true   error, use get_last_error()
-                   to see the error number.
-  */
-  bool execute_direct(LEX_STRING sql_text);
-
-  /**
-    Same as the previous, but takes an instance of Server_runnable
-    instead of SQL statement text.
-
-    @return execution status
-
-    @retval  false  success, use get_field_count()
-                    if your code fragment is supposed to
-                    return a result set
-    @retval  true   failure
-  */
-  bool execute_direct(Server_runnable *server_runnable);
-
-  /**
-    The following three members are only valid if execute_direct()
-    or move_to_next_result() returned an error.
-    They never fail, but if they are called when there is no
-    result, or no error, the result is not defined.
-  */
-  const char *get_last_error() const {
-    return m_diagnostics_area.message_text();
-  }
-
-  unsigned int get_last_errno() const {
-    return m_diagnostics_area.mysql_errno();
-  }
-
-  Ed_result_set *get_result_sets() { return m_rsets; }
-
-  ~Ed_connection() { free_old_result(); }
-
- private:
-  Diagnostics_area m_diagnostics_area;
-  /**
-    Execute direct interface does not support multi-statements, only
-    multi-results. So we never have a situation when we have
-    a mix of result sets and OK or error packets. We either
-    have a single result set, a single error, or a single OK,
-    or we have a series of result sets, followed by an OK or error.
-  */
-  THD *m_thd;
-  Ed_result_set *m_rsets;
-  Ed_result_set *m_current_rset;
-  friend class Protocol_local;
-
- private:
-  void free_old_result();
-  void add_result_set(Ed_result_set *ed_result_set);
-
- private:
-  Ed_connection(const Ed_connection &);      /* not implemented */
-  Ed_connection &operator=(Ed_connection &); /* not implemented */
-};
-
-/** One result set column. */
-
-struct Ed_column final : public LEX_STRING {
-  /** Implementation note: destructor for this class is never called. */
-};
-
-/** One result set record. */
-
-class Ed_row final {
- public:
-  const Ed_column &operator[](const unsigned int column_index) const {
-    return *get_column(column_index);
-  }
-  const Ed_column *get_column(const unsigned int column_index) const {
-    assert(column_index < size());
-    return m_column_array + column_index;
-  }
-  size_t size() const { return m_column_count; }
-
-  Ed_row(Ed_column *column_array_arg, size_t column_count_arg)
-      : m_column_array(column_array_arg), m_column_count(column_count_arg) {}
-
- private:
-  Ed_column *m_column_array;
-  size_t m_column_count; /* TODO: change to point to metadata */
-};
-
 class Server_side_cursor;
 
 /**
@@ -440,8 +246,58 @@ class Prepared_statement final {
 #ifdef HAVE_PSI_PS_INTERFACE
   PSI_prepared_stmt *get_PS_prepared_stmt() { return m_prepared_stmt; }
 #endif
+
+  /**
+    Assign parameter values from the execute packet.
+
+    @param thd             current thread
+    @param expanded_query  a container with the original SQL statement.
+                           '?' placeholders will be replaced with
+                           their values in case of success.
+                           The result is used for logging and replication
+    @param has_new_types   flag used to signal that new types are provided.
+    @param parameters      prepared statement's parsed parameters.
+
+    @returns false if success, true if error
+             (likely a conversion error, out of memory, or malformed packet)
+  */
   bool set_parameters(THD *thd, String *expanded_query, bool has_new_types,
                       PS_PARAM *parameters);
+
+  enum enum_param_pack_type {
+    /*
+     Parameter values are coming from client over network (vio).
+     */
+    PACKED,
+
+    /*
+     Parameter values are sent either using using Server_component service
+     API's or Plugins or directly from SQL layer modules.
+
+     UNPACKED means that the parameter value buffer points to MYSQL_TIME*
+     */
+    UNPACKED
+  };
+
+  /**
+    Assign parameter values from the execute packet.
+
+    @param thd             current thread
+    @param expanded_query  a container with the original SQL statement.
+                           '?' placeholders will be replaced with
+                           their values in case of success.
+                           The result is used for logging and replication
+    @param has_new_types   flag used to signal that new types are provided.
+    @param parameters      prepared statement's parsed parameters.
+    @param param_pack_type parameters pack type.
+
+    @returns false if success, true if error
+             (likely a conversion error, out of memory, or malformed packet)
+  */
+  bool set_parameters(THD *thd, String *expanded_query, bool has_new_types,
+                      PS_PARAM *parameters,
+                      enum enum_param_pack_type param_pack_type);
+
   bool set_parameters(THD *thd, String *expanded_query);
   void trace_parameter_types(THD *thd);
   void close_cursor();
@@ -460,7 +316,8 @@ class Prepared_statement final {
   bool insert_parameters_from_vars(THD *thd, List<LEX_STRING> &varnames,
                                    String *query);
   bool insert_parameters(THD *thd, String *query, bool has_new_types,
-                         PS_PARAM *parameters);
+                         PS_PARAM *parameters,
+                         enum enum_param_pack_type param_pack_type);
 };
 
 #endif  // SQL_PREPARE_H
