@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2023, Oracle and/or its affiliates.
+Copyright (c) 1995, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -139,6 +140,10 @@ enum class Fil_state {
   /** Space ID matches but the paths don't match. */
   MOVED,
 
+  /** Space ID and paths match but dd_table data dir flag doesn't
+  exist or tablespace is created outside default data dir */
+  MOVED_PREV_OR_HAS_DATADIR,
+
   /** Tablespace and/or filename was renamed. The DDL log will handle
   this case. */
   RENAMED
@@ -697,6 +702,14 @@ class Fil_path {
   @return true if this path is the same as the other path */
   [[nodiscard]] bool is_same_as(const std::string &other) const;
 
+  /** Get the absolute directory of this path */
+  [[nodiscard]] Fil_path get_abs_directory() const;
+
+  /** Check if the directory to path is same as directory as the other path.
+  @param[in]  other  directory path to compare to
+  @return true if this path directory is the same as the other path directory */
+  [[nodiscard]] bool is_dir_same_as(const Fil_path &other) const;
+
   /** Check if two path strings are equal. Put them into Fil_path objects
   so that they can be compared correctly.
   @param[in]  first   first path to check
@@ -2078,10 +2091,11 @@ inline fil_space_t *fil_space_get_sys_space() {
 @param[in]      parse_only      Don't apply, parse only
 @return pointer to next redo log record
 @retval nullptr if this log record was truncated */
-[[nodiscard]] byte *fil_tablespace_redo_create(byte *ptr, const byte *end,
-                                               const page_id_t &page_id,
-                                               ulint parsed_bytes,
-                                               bool parse_only);
+[[nodiscard]] const byte *fil_tablespace_redo_create(const byte *ptr,
+                                                     const byte *end,
+                                                     const page_id_t &page_id,
+                                                     ulint parsed_bytes,
+                                                     bool parse_only);
 
 /** Redo a tablespace delete.
 @param[in]      ptr             redo log record
@@ -2091,10 +2105,11 @@ inline fil_space_t *fil_space_get_sys_space() {
 @param[in]      parse_only      Don't apply, parse only
 @return pointer to next redo log record
 @retval nullptr if this log record was truncated */
-[[nodiscard]] byte *fil_tablespace_redo_delete(byte *ptr, const byte *end,
-                                               const page_id_t &page_id,
-                                               ulint parsed_bytes,
-                                               bool parse_only);
+[[nodiscard]] const byte *fil_tablespace_redo_delete(const byte *ptr,
+                                                     const byte *end,
+                                                     const page_id_t &page_id,
+                                                     ulint parsed_bytes,
+                                                     bool parse_only);
 
 /** Redo a tablespace rename.
 This function doesn't do anything, simply parses the redo log record.
@@ -2105,10 +2120,11 @@ This function doesn't do anything, simply parses the redo log record.
 @param[in]      parse_only      Don't apply, parse only
 @return pointer to next redo log record
 @retval nullptr if this log record was truncated */
-[[nodiscard]] byte *fil_tablespace_redo_rename(byte *ptr, const byte *end,
-                                               const page_id_t &page_id,
-                                               ulint parsed_bytes,
-                                               bool parse_only);
+[[nodiscard]] const byte *fil_tablespace_redo_rename(const byte *ptr,
+                                                     const byte *end,
+                                                     const page_id_t &page_id,
+                                                     ulint parsed_bytes,
+                                                     bool parse_only);
 
 /** Redo a tablespace extend
 @param[in]      ptr             redo log record
@@ -2118,10 +2134,11 @@ This function doesn't do anything, simply parses the redo log record.
 @param[in]      parse_only      Don't apply the log if true
 @return pointer to next redo log record
 @retval nullptr if this log record was truncated */
-[[nodiscard]] byte *fil_tablespace_redo_extend(byte *ptr, const byte *end,
-                                               const page_id_t &page_id,
-                                               ulint parsed_bytes,
-                                               bool parse_only);
+[[nodiscard]] const byte *fil_tablespace_redo_extend(const byte *ptr,
+                                                     const byte *end,
+                                                     const page_id_t &page_id,
+                                                     ulint parsed_bytes,
+                                                     bool parse_only);
 
 /** Parse and process an encryption redo record.
 @param[in]      ptr             redo log record
@@ -2129,9 +2146,10 @@ This function doesn't do anything, simply parses the redo log record.
 @param[in]      space_id        the tablespace ID
 @param[in]      lsn             lsn for REDO record
 @return log record end, nullptr if not a complete record */
-[[nodiscard]] byte *fil_tablespace_redo_encryption(byte *ptr, const byte *end,
-                                                   space_id_t space_id,
-                                                   lsn_t lsn);
+[[nodiscard]] const byte *fil_tablespace_redo_encryption(const byte *ptr,
+                                                         const byte *end,
+                                                         space_id_t space_id,
+                                                         lsn_t lsn);
 
 /** Read the tablespace id to path mapping from the file
 @param[in]      recovery        true if called from crash recovery */
@@ -2155,23 +2173,26 @@ bool fil_update_partition_name(space_id_t space_id, uint32_t fsp_flags,
                                std::string &dd_path);
 
 /** Add tablespace to the set of tablespaces to be updated in DD.
-@param[in]      dd_object_id    Server DD tablespace ID
-@param[in]      space_id        Innodb tablespace ID
-@param[in]      space_name      New tablespace name
-@param[in]      old_path        Old Path in the data dictionary
-@param[in]      new_path        New path to be update in dictionary */
+@param[in]    dd_object_id                   Server DD tablespace ID
+@param[in]    space_id                       InnoDB tablespace ID
+@param[in]    space_name                     Tablespace name
+@param[in]    old_path                       Old Path in the data dictionary
+@param[in]    new_path                       New path to be update in dictionary
+@param[in]    moved_prev_or_has_datadir      The move has happened before
+                                             8.0.38/8.4.1/9.0.0 or table is
+                                             created with data dir clause.*/
 void fil_add_moved_space(dd::Object_id dd_object_id, space_id_t space_id,
                          const char *space_name, const std::string &old_path,
-                         const std::string &new_path);
-
+                         const std::string &new_path,
+                         bool moved_prev_or_has_datadir);
 /** Lookup the tablespace ID and return the path to the file. The filename
 is ignored when testing for equality. Only the path up to the file name is
 considered for matching: e.g. ./test/a.ibd == ./test/b.ibd.
-@param[in]  space_id      tablespace ID to lookup
-@param[in]  space_name    tablespace name
-@param[in]  fsp_flags     tablespace flags
-@param[in]  old_path      the path found in dd:Tablespace_files
-@param[out] new_path      the scanned path for this space_id
+@param[in]  space_id                tablespace ID to lookup
+@param[in]  space_name              tablespace name
+@param[in]  fsp_flags               tablespace flags
+@param[in]  old_path                the path found in dd:Tablespace_files
+@param[out] new_path                the scanned path for this space_id
 @return status of the match. */
 [[nodiscard]] Fil_state fil_tablespace_path_equals(space_id_t space_id,
                                                    const char *space_name,
