@@ -393,9 +393,9 @@ class NdbOperation {
    *                    the attribute, or a NULL pointer
    *                    (indicating error).
    */
-  NdbRecAttr *getValue(const char *anAttrName, char *aValue = 0);
-  NdbRecAttr *getValue(Uint32 anAttrId, char *aValue = 0);
-  NdbRecAttr *getValue(const NdbDictionary::Column *, char *val = 0);
+  NdbRecAttr *getValue(const char *anAttrName, char *aValue = nullptr);
+  NdbRecAttr *getValue(Uint32 anAttrId, char *aValue = nullptr);
+  NdbRecAttr *getValue(const NdbDictionary::Column *, char *val = nullptr);
 
   /**
    * Define an attribute to set or update in query.
@@ -906,9 +906,6 @@ class NdbOperation {
    *
    * @return method number where the error occurred.
    */
-#ifndef DOXYGEN_SHOULD_SKIP_DEPRECATED
-  int getNdbErrorLine();
-#endif
   int getNdbErrorLine() const;
 
   /**
@@ -1317,7 +1314,8 @@ class NdbOperation {
    ******************************************************************************/
 
   virtual int equal_impl(const NdbColumnImpl *, const char *aValue);
-  virtual NdbRecAttr *getValue_impl(const NdbColumnImpl *, char *aValue = 0);
+  virtual NdbRecAttr *getValue_impl(const NdbColumnImpl *,
+                                    char *aValue = nullptr);
   NdbRecAttr *getValue_NdbRecord(const NdbColumnImpl *tAttrInfo, char *aValue);
   int setValue(const NdbColumnImpl *anAttrObject, const char *aValue);
   NdbBlob *getBlobHandle(NdbTransaction *aCon,
@@ -1593,12 +1591,6 @@ inline int NdbOperation::checkMagicNumber(bool b) {
 
 inline void NdbOperation::setStartIndicator() { theStartIndicator = 1; }
 
-inline int NdbOperation::getNdbErrorLine() {
-  // delegate to overloaded const function for same semantics
-  const NdbOperation *const cthis = this;
-  return cthis->NdbOperation::getNdbErrorLine();
-}
-
 inline int NdbOperation::getNdbErrorLine() const { return theErrorLine; }
 
 /******************************************************************************
