@@ -27,6 +27,8 @@
 #include <my_sys.h>
 #include "client/pattern_matcher.cc"
 
+struct CHARSET_INFO;
+
 namespace mysql_client_test_ns {
 
 class PatternMatcherTest : public ::testing::Test {
@@ -35,8 +37,7 @@ class PatternMatcherTest : public ::testing::Test {
 
  protected:
   void SetUp() override {
-    MY_CHARSET_LOADER loader;
-    cs_info = my_collation_get_by_name(&loader, "utf8mb4_0900_ai_ci", MYF(0));
+    cs_info = get_charset_by_name("utf8mb4_0900_ai_ci", MYF(0));
   }
 };
 
